package org.jbpm.jpdl.el.impl;


import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.dom4j.Document;
import org.dom4j.DocumentHelper;
import org.dom4j.Element;
import org.dom4j.Namespace;
import org.dom4j.XPath;
import org.dom4j.xpath.DefaultXPath;
import org.jbpm.JbpmConfiguration;
import org.jbpm.JbpmException;
import org.jbpm.graph.exe.ExecutionContext;
import org.jbpm.jpdl.el.ELException;
import org.jbpm.jpdl.el.ExpressionEvaluator;
import org.jbpm.jpdl.el.VariableResolver;
import org.jbpm.jpdl.el.FunctionMapper;

public class JbpmExpressionEvaluator {

  static ExpressionEvaluator evaluator = new ExpressionEvaluatorImpl();
  public static void setExpressionEvaluator(ExpressionEvaluator expressionEvaluator) {
    JbpmExpressionEvaluator.evaluator = expressionEvaluator;
  }
  
  static VariableResolver variableResolver = null;
  public static void setVariableResolver(VariableResolver variableResolver) {
    JbpmExpressionEvaluator.variableResolver = variableResolver;
  }
  
  static FunctionMapper functionMapper = null;
  public static void setFunctionMapper(FunctionMapper functionMapper) {
    JbpmExpressionEvaluator.functionMapper = functionMapper;
  }
  
  public static Object evaluate(String expression, ExecutionContext executionContext) {
    return evaluate(expression, executionContext, getUsedVariableResolver(), getUsedFunctionMapper());
  }

  public static Object evaluate(String expression, ExecutionContext executionContext, VariableResolver usedVariableResolver, FunctionMapper functionMapper) {
	  if (expression.startsWith("#{xpath(")){
		  return xpathEvaluate(expression, executionContext, usedVariableResolver, functionMapper);
	  }else{
		  return jbpmEvaluate(expression, executionContext, usedVariableResolver, functionMapper);
	  }
  }
  public static Object xpathEvaluate(String expression, ExecutionContext executionContext, VariableResolver usedVariableResolver, FunctionMapper functionMapper) {	
		try{
			String varName = expression.substring(expression.indexOf('(')+1, expression.indexOf(','));
//			String xPath = expression.substring(expression.indexOf(",")+1, expression.indexOf(")"));
			String xPath = expression.substring(expression.indexOf(",")+1, expression.lastIndexOf(')'));

		
			String xml = (String)executionContext.getVariable(varName);
			Document document = DocumentHelper.parseText(xml);
		
			List<Namespace> namespaces = new ArrayList<Namespace>();
			retrieveNamespace(document.getRootElement(),namespaces);
		
			Map<String, String> namespaceMap = new HashMap<String, String>();
			for (Namespace n : namespaces){
				namespaceMap.put(n.getPrefix(), n.getURI());
			}
		
			XPath xpath = new DefaultXPath(xPath);
			xpath.setNamespaceURIs(namespaceMap);
			return xpath.evaluate(document);
			
		}catch (Exception e) {
			e.printStackTrace();
			return false;
		}
  }
  
  
  private static  void retrieveNamespace(Element el, List l){
		l.addAll(el.declaredNamespaces());
		if (!isLeaf(el)){
			List<Element> childs = el.elements();
			for ( Element e : childs)
				retrieveNamespace(e, l);
		}
  }
  
  public static boolean isLeaf(Element elem){
		return elem.elements().size() == 0;
  }
  
  
  public static Object jbpmEvaluate(String expression, ExecutionContext executionContext, VariableResolver usedVariableResolver, FunctionMapper functionMapper) {	
	Object result = null;
    
    ExecutionContext.pushCurrentContext(executionContext);
    try {
      String dollarExpression = translateExpressionToDollars(expression);
      result = evaluator.evaluate(dollarExpression, Object.class, usedVariableResolver, functionMapper);

    } catch (ELException e) {
      
      throw new JbpmException("couldn't evaluate expression '"+expression+"'", (e.getRootCause()!=null ? e.getRootCause() : e));
    } finally {
      ExecutionContext.popCurrentContext(executionContext);
    }
    
    return result;
  }

  static String translateExpressionToDollars(String expression) {
    int hashIndex = expression.indexOf("#{");
    if (hashIndex == -1) return expression;
    
    char[] expressionChars = expression.toCharArray();
    do {
      expressionChars[hashIndex] = '$';
      hashIndex = expression.indexOf("#{", hashIndex + 2);
    } while (hashIndex != -1);
    return new String(expressionChars);
  }

  public static VariableResolver getUsedVariableResolver() {
    if (variableResolver!=null) {
      return variableResolver;
    }
    return (VariableResolver) JbpmConfiguration.Configs.getObject("jbpm.variable.resolver");
  }

  public static FunctionMapper getUsedFunctionMapper() {
    if (functionMapper!=null) {
      return functionMapper;
    }
    if (JbpmConfiguration.Configs.hasObject("jbpm.function.mapper")) {
      return (FunctionMapper) JbpmConfiguration.Configs.getObject("jbpm.function.mapper");
    }
    return null;
  }
}

