/**

    Copyright 2007, 2009 Engineering Ingegneria Informatica S.p.A.

    This file is part of Spagic.

    Spagic is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation; either version 3 of the License, or
    any later version.

    Spagic is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
    
 **/
package org.spagic.workflow.api;



public interface IControlAPI {

	/**
	 * Start a new process instance given the process name ( process definition
	 * name )
	 * 
	 * @param processName
	 *          - the process id
	 * @return the newly created instance process instance id
	 */
	public Long startByProcessName(String processName);

	/**
	 * Start a new process instance given the process name ( process definition name )
	 * 
	 * @param processName - the process id 
	 * @return the newly created instance process instance id
	 */
	public Long startByProcessName(String processName, Variable[] vars);
	
	/**
	 * Abort the process instance given the process name ( processInstanceId )
	 * 
	 * @param processInstanceId
	 *          - the process instance Id
	 */
	public void abort(long processInstanceId);
	
	/**
	 * Restart the process instance given the process instance id ( processInstanceId )
	 * 
	 * @param processInstanceId - The process instance Id
	 * @param modifiedMessages - The messages changed by the user
	 */
	public void restart(long processInstanceId, ModifiedMessage[] modifiedMessages);

	
	/** NOT FINISHED SO NOT PUBLISH IN PUBLIC API
	 * Set a flag for process instance, during process execution if an automatic
	 * task find the process
	 * 
	 * @param processInstanceId - the process instance Id
	 * @deprecated
	 * TO REVIEW AND NOT YET IMPLEMENTED
	
	public void stopOnAutomaticTask(long processInstanceId, boolean stop);
	 */

	
	
	/**
	 * Execute the task called "task" in the instance identified by
	 * "processInstanceId", passing a set of variables.
	 * 
	 * @param processInstanceId
	 *          - The process Instance Id
	 * @param taskName
	 *          - The task to execute
	 * @param vars
	 *          - the array of variables, set to null if you don't want pass
	 *          variables
	 */
	public void executeTask(long processInstanceId, String taskName, Variable[] vars);
	
	/**
	 * Start the task called "taskName" in the instance identified by
	 * "processInstanceId", passing a set of variables. The difference with this method is
	 * that this one does not complete the task
	 * 
	 * @param processInstanceId
	 *          - The process Instance Id
	 * @param taskName
	 *          - The task to execute
	 * @param vars
	 *          - the array of variables, set to null if you don't want pass
	 *          variables
	 */
	public void startTask(long processInstanceId, String taskName, Variable[] vars);

	/**
	 * Execute the task called "task" specifying the "task status" in the instance
	 * identified by "processInstanceId", passing a set of variables.
	 * 
	 * @param processInstanceId
	 *          - The process Instance Id
	 * @param taskName
	 *          - The task to execute
	 * @param taskStatus
	 *          - The status of the task
	 * @param vars
	 *          - the array of variables, set to null if you don't want pass
	 *          variables
	 */
	public void executeTaskWithStatus(long processInstanceId, String taskName, String taskStatus, Variable[] vars);

	/**
	 * Execute the task called identified by "taskId" in the instance identified
	 * by "processInstanceId", passing a set of variables.
	 * 
	 * @param processInstanceId
	 *          - The process Instance Id
	 * @param taskId
	 *          - The task id
	 * @param vars
	 *          - the array of variables, set to null if you don't want pass
	 *          variables
	 */
	public void executeTask(long processInstanceId, long taskId, Variable[] vars);

	/**
	 * Start the task called identified by "taskId" in the instance identified
	 * by "processInstanceId", passing a set of variables.
	 * 
	 * @param processInstanceId
	 *          - The process Instance Id
	 * @param taskId
	 *          - The task id
	 * @param vars
	 *          - the array of variables, set to null if you don't want pass
	 *          variables
	 */
	public void startTask(long processInstanceId, long taskId, Variable[] vars);

	/**
	 * Execute the task called identified by "taskId" in the instance identified
	 * by "processInstanceId" specifying "TaskStatus", passing a set of variables.
	 * 
	 * @param processInstanceId
	 *          - The process Instance Id
	 * @param taskId
	 *          - The task id
	 * @param taskStatus
	 *          - The status of the task
	 * @param vars
	 *          - the array of variables, set to null if you don't want pass
	 *          variables
	 */
	public void executeTaskWithStatus(long processInstanceId, long taskId, String taskStatus, Variable[] vars);

	/**
	 * Execute the task identified by "taskId" by the given "actor" in the
	 * instance identified by "processInstanceId", passing a set of variables.
	 * 
	 * @param actor
	 *          - The actor performing the task
	 * @param processInstanceId
	 *          - The process Instance Id
	 * @param taskId
	 *          - The task id
	 * @param vars
	 *          - the array of variables, set to null if you don't want pass
	 *          variables
	 */
	public void executeTaskByActor(String actor, long processInstanceId, long taskId, Variable[] vars);

	/**
	 * Start the task identified by "taskId" by the given "actor" in the
	 * instance identified by "processInstanceId", passing a set of variables.
	 * 
	 * @param actor
	 *          - The actor performing the task
	 * @param processInstanceId
	 *          - The process Instance Id
	 * @param taskId
	 *          - The task id
	 * @param vars
	 *          - the array of variables, set to null if you don't want pass
	 *          variables
	 */
	public void startTaskByActor(String actor, long processInstanceId, long taskId, Variable[] vars);

	/**
	 * Execute the task called "task" by the given "actor" in the instance
	 * identified by "processInstanceId", passing a set of variables.
	 * 
	 * @param actor
	 *          - The actor performing the task
	 * @param processInstanceId
	 *          - The process Instance Id
	 * @param taskName
	 *          - The task to execute
	 * @param vars
	 *          - the array of variables, set to null if you don't want pass
	 *          variables
	 */
	public void executeTaskByActor(String actor, long processInstanceId, String taskName, Variable[] vars);

	/**
	 * Start the task called "task" by the given "actor" in the instance
	 * identified by "processInstanceId", passing a set of variables.
	 * 
	 * @param actor
	 *          - The actor performing the task
	 * @param processInstanceId
	 *          - The process Instance Id
	 * @param taskName
	 *          - The task to execute
	 * @param vars
	 *          - the array of variables, set to null if you don't want pass
	 *          variables
	 */
	public void startTaskByActor(String actor, long processInstanceId, String taskName, Variable[] vars);
	

	/**
	 * Update the task instance status saving the completion percentage.
	 * @param processInstanceId Process instance id
	 * @param taskName The task to update
	 * @param completionPercentage New completion percentage
	 */
	public void advanceTaskCompletion(long processInstanceId, String taskName, int completionPercentage);

	/**
	 * Update the task instance status saving the completion percentage.
	 * @param processInstanceId Process instance id
	 * @param taskId The task instance identifier
	 * @param completionPercentage New completion percentage
	 */
	public void advanceTaskCompletion(long processInstanceId, long taskId, int completionPercentage);
	

	/**
	 * Reject a task instance, undoing a previously accepted task
	 * @param processInstanceId Process instance id
	 * @param taskId Task instance id
	 */
	public void rejectTask(long processInstanceId, long taskId);
	
	/**
	 * Reject a task instance, undoing a previously accepted task
	 * @param processInstanceId Process instance id
	 * @param taskId Task instance id
	 */
	public void rejectTaskByActor(String actorId, long processInstanceId, long taskId);
	

	/**
	 * Set a value of a global variable on process instance.
	 * 
	 * @param processInstanceId
	 * @param variableName
	 *          - the name of variable to set
	 * @param variableValue
	 *          - the value of the variable to set
	 */
	public void setGlobalVariable(long processInstanceId, String variableName, Object variableValue);

	/**
	 * Get the value of the specified global variables on process instance.
	 * 
	 * @param processInstanceId
	 * @param variableName
	 *          - the name of variable to set
	 */
	public Variable getGlobalVariable(long processInstanceId, String variableName);

	/**
	 * Get the values of all global variables on process instance.
	 * 
	 * @param processInstanceId
	 */
	public Variable[] getGlobalVariables(long processInstanceId);

	/**
	 * Set the values of some global variables on process instance.
	 * 
	 * @param processInstanceId
	 *          Process instance id
	 * @param variables
	 *          Variables values
	 */
	public void setGlobalVariables(long processInstanceId, Variable[] variables);

	/**
	 * Establish dinamically a link between two different process instances.
	 * 
	 * @param firstProcessInstanceId
	 * @param secondProcessInstanceId
	 */
	public void linkProcesses(long firstProcessInstanceId, long secondProcessInstanceId);

	/**
	 * Get all the variables assigned to a specific task on a process instance.
	 * 
	 * @param processInstanceId
	 * @param taskName
	 *          the name of the task
	 */
	public Variable[] getTaskVariables(long processInstanceId, String taskName);

	/**
	 * Get a variable assigned to a specific task on a process instance.
	 * 
	 * @param processInstanceId
	 * @param taskName
	 *          the name of the task
	 * @param variableName
	 *          - the name of variable to set
	 */
	public Variable getTaskVariable(long processInstanceId, String taskName, String variableName);

	/**
	 * Set a value of a "task" variable on process instance.
	 * 
	 * @param processInstanceId
	 * @param taskName
	 *          the name of the task
	 * @param variableName
	 *          - the name of variable to set
	 * @param variableValue
	 *          - the value of the variable to set
	 */
	public void setTaskVariable(long processInstanceId, String taskName, String variableName, Object variableValue);

	/**
	 * Set the values of some task variables on process instance.
	 * 
	 * @param processInstanceId
	 *          Process instance id
	 * @param variables
	 *          Variables values
	 */
	public void setTaskVariables(long processInstanceId, String taskName, Variable[] variables);
	
	public void advanceTaskCompletionByActor(String actorId,long processInstanceId, long taskId, int completionPercentage);
	
	public void advanceTaskCompletionByActor(String actorId,long processInstanceId, String taskName, int completionPercentage);
}
