package org.spagic3.attachments.db;

import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.sql.Blob;
import java.sql.SQLException;

import javax.activation.DataHandler;
import javax.mail.util.ByteArrayDataSource;

import org.hibernate.Hibernate;
import org.hibernate.HibernateException;
import org.hibernate.Query;
import org.hibernate.Session;
import org.hibernate.SessionFactory;
import org.hibernate.Transaction;
import org.hibernate.cfg.Configuration;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.spagic3.attachments.IAttachmentStore;
import org.spagic3.attachments.model.Attachment;

public class DBAttachmentStore implements IAttachmentStore {

	private static Logger logger = 
		LoggerFactory.getLogger(DBAttachmentStore.class);

	private final SessionFactory sessionFactory;
	public DBAttachmentStore () {
		try {
			sessionFactory = new Configuration().configure()
					.buildSessionFactory();
		} catch (Throwable ex) {
			// Make sure you log the exception, as it might be swallowed
			logger.error("Initial SessionFactory creation failed.", ex);
			throw new ExceptionInInitializerError(ex);
		}
	}

	@Override
	public void delete(String id) {
		Session session = sessionFactory.openSession();

		Transaction transact = null;
		try {
			transact = session.beginTransaction();
			Query killer = session.createQuery("delete from Attachment where id = :id");
			killer.setLong("id", Long.parseLong(id));
			killer.executeUpdate();
			transact.commit();

		}
		catch(HibernateException hex) {
			logger.error("Attachment deletion failure",hex);
			if(transact !=null)
				transact.rollback();
			throw new RuntimeException("Attachment deletion failure",hex);
		}
		catch(Throwable thr) {
			logger.error("Attachment deletion failure - Unexpected exception",thr);
			if(transact !=null)
				transact.rollback();
			throw new RuntimeException("Attachment deletion failure - Unexpected exception",thr);
		}
		finally {
			try {
				session.close();
			}
			catch(HibernateException hex) {
				logger.error("Session close failure - swallowed",hex);
			}
		}

	}

	@Override
	public DataHandler load(String id) {

		Session session = sessionFactory.openSession();
		Attachment attachment = null;
		Transaction transaction = null;
		try {
			transaction = session.beginTransaction();
			attachment = (Attachment) session.load(Attachment.class, Long.parseLong(id));
			
			Blob b = attachment.getData();
			int blen = (int) b.length();

			ByteArrayOutputStream baos = new ByteArrayOutputStream();

			baos.write(b.getBytes(1, blen), 0, blen);
			transaction.commit();
			transaction = null;

			baos.flush();
			baos.close();

			ByteArrayDataSource bads = 
				new ByteArrayDataSource(baos.toByteArray(), 
										"application/octet-stream");

			DataHandler dha = new DataHandler(bads);
			return dha;
		} 
		catch (IOException ioe) {
 			try {
 				if(transaction != null)
 					transaction.rollback();
 			}
			catch (Throwable thrRollback) {
				logger.error("Unexpected Throwable doing rollback on select - swallowed.",thrRollback);
			}
			logger.error("IOException handling the BLOB",ioe);
			throw new RuntimeException("IOException handling the BLOB",ioe);
		}
		catch (SQLException sex) {
 			try {
 				if(transaction != null)
 					transaction.rollback();
 			}
			catch (Throwable thrRollback) {
				logger.error("Unexpected Throwable doing rollback on select - swallowed.",thrRollback);
			}
			logger.error("SQLException handling the BLOB",sex);
			throw new RuntimeException("SQLException handling the BLOB",sex);
		} 
		finally {
			try {
				session.close();
			}
			catch(HibernateException hex) {
				logger.error("Session close failure - swallowed",hex);
			}
		}


        
	}


	@Override
	public String store(String name, String correlationID, DataHandler value) {
		Session session = sessionFactory.openSession();
		Attachment attachment = new Attachment();

		try {
			ByteArrayOutputStream baos = new ByteArrayOutputStream();

			InputStream is = value.getInputStream();

			byte buf[] = new byte[4096];
			int read;

			while( (read = is.read(buf)) > -1 ) {
				baos.write(buf, 0, read);
			}
			buf = null;

			attachment.setData(Hibernate.createBlob(baos.toByteArray()));
			attachment.setCorrelationId(correlationID);
			baos.close();
			baos = null;
		}
		catch (IOException ioe) {
			logger.error("IOException handling the BLOB",ioe);
			throw new RuntimeException("IOException handling the BLOB",ioe);
		}

		Transaction transact = session.beginTransaction();

		try {
			session.save(attachment);
			transact.commit();
			
			return "" + attachment.getId();
		}
 		catch (Throwable rex) {
 			try {
 				transact.rollback();
 			}
			catch (Throwable thr) {
				logger.error("Unexpected Throwable doing rollback on insert - swallowed.",thr);
			}
 			logger.error("RuntimeException handling the BLOB",rex);
 			throw new RuntimeException("Unexpected Throwable handling the BLOB",rex);
 		}
		finally {
			try {
				session.close();
			}
			catch(HibernateException hex) {
				logger.error("Session close failure - swallowed",hex);
			}
		}

	}

	private String attachmentStoreId = null;
	
	@Override
	public String getAttachmentStoreId() {
		return attachmentStoreId;
	}

}
