package org.spagic3.client.api;

import java.util.HashMap;
import java.util.Map;


/**
 * Class <code>ClientMessage</code> A message sent throught the Client API
 *
 * @author <a href="mailto:Gianfranco.Boccalon@eng.it">Gianfranco Boccalon</a>
 * @version $Revision$
 */
public class ClientMessage {

	private String id = null;
	private String body = null;
	private Map<String, String> properties = null;
	private Map<String, IAttachment> attachments = null;

	/**
	 * Creates a new instance of <code>ClientMessage</code> .
	 *
	 * @param id a <code>String</code> ID of the message
	 * @param body a <code>String</code> body of the message
	 * @param properties a <code>Map<String, String></code> with the
	 * mesage properties
	 */
	public ClientMessage(String id, String body, Map<String, String> properties){
		this.body = body;
		this.properties = properties;
	}

	/**
	 * Creates a new instance of <code>ClientMessage</code> .
	 *
	 * @param id a <code>String</code> ID of the message
	 */
	public ClientMessage(String id){
		this(id, null, null);
	}

	/**
	 * Creates a new instance of <code>ClientMessage</code> .
	 *
	 * @param id a <code>String</code> ID of the message
	 * @param body a <code>String</code> body of the message
	 */
	public ClientMessage(String id, String body){
		this(id, body, null);
	}

	/**
	 * <code>getId</code> returns the message ID
	 *
	 * @return a <code>String</code> the message ID
	 */
	public String getId() {
		return id;
	}

	/**
	 * <code>setId</code> sets the new message ID
	 *
	 * @param id a <code>String</code>  with the new message ID
	 */
	public void setId(String id) {
		this.id = id;
	}

	/**
	 * <code>setBody</code> sets the message body
	 *
	 * @param body a <code>String</code> with the message new body
	 */
	public void setBody(String body){
		this.body = body;
	}
	/**
	 * <code>setProperties</code> set the message properties
	 *
	 * @param properties a <code>Map<String, String></code> with the
	 * new mesage properties
	 */
	public void setProperties(Map<String, String> properties){
		this.properties = properties;
	}

	/**
	 * <code>getBody</code> gets the body of hte message
	 *
	 * @return a <code>String</code> the body of hte message
	 */
	public String getBody(){
		return this.body;
	}

	public Map<String, String> getProperties(){
		return this.properties;
	}

	/**
	 * <code>setProperty</code> Sets a message property
	 *
	 * @param key a <code>String</code> with the property name
	 * @param value a <code>String</code> with the property value
	 */
	public void setProperty(String key, String value){
		if (properties == null)
			this.properties = new HashMap<String, String>();
		properties.put(key,value);
	}

	/**
	 * <code>setAttachment</code> Adds an attachment to the message
	 *
	 * @param key a <code>String</code> the name (identifier) of the
	 * attachment
	 * @param dh an <code>IAttachment</code> the attachments
	 * themselves
	 */
	public void setAttachment(String key, IAttachment dh){
		if (attachments == null)
			this.attachments = new HashMap<String, IAttachment>();
		attachments.put(key,dh);
	}

	/**
	 * <code>setAttachment</code> returns the attachments
	 * <b>converting</b> each attacment into a DataHandler for
	 * processing inside the OSGi engine.
	 *
	 * @param key a <code>String</code> the name (identifier) of the
	 * attachment
	 * @param dh an <code>IAttachment</code> the new map with the
	 * attachments converted into a DataHandler
	 */
	public Map<String, IAttachment> getAttachments() {
		
		return attachments;
	}

	/**
	 * <code>setAttachments</code> Replace the attacments of the message
	 *
	 * @param attacments a <code>Map<String, IAttachment></code> with
	 * the new set of attachments.
	 */
	public void setAttachments(Map<String, IAttachment> attachments) {
		this.attachments = attachments;
	}
}
