package org.spagic3.components.bpm.deploy;

import java.io.File;
import java.io.FileInputStream;
import java.util.Date;
import java.util.List;
import java.util.Properties;

import org.dom4j.Document;
import org.eclipse.emf.ecore.resource.Resource;
import org.eclipse.emf.ecore.resource.ResourceSet;
import org.eclipse.emf.ecore.resource.impl.ResourceSetImpl;
import org.eclipse.emf.ecore.xmi.XMIResource;
import org.eclipse.stp.im.BasicProperty;
import org.eclipse.stp.im.ImFactory;
import org.eclipse.stp.im.ImPackage;
import org.eclipse.stp.im.ProcessCollection;
import org.eclipse.stp.im.Property;
import org.eclipse.stp.im.Step;
import org.eclipse.stp.im.StpIntermediateModel;
import org.eclipse.stp.im.constants.ImConstants;
import org.hibernate.Session;
import org.jbpm.JbpmContext;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.spagic.metadb.model.SchemaMetadata;
import org.spagic.workflow.api.jbpm.ISpagicJBPMCommand;
import org.spagic.workflow.api.jbpm.ProcessEngine;
import org.spagic.workflow.api.jbpm.ProcessInstanceExecutionContext;
import org.spagic3.components.bpm.deploy.utils.DeploymentUtils;
import org.spagic3.constants.SpagicConstants;

public class DeploymentHelper {
	
	private String process = null;
	private static Logger logger = LoggerFactory.getLogger(DeploymentHelper.class);
	
	public DeploymentHelper(String process) {
		super();
		this.process = process;
	}

	@SuppressWarnings("restriction")
	protected StpIntermediateModel getImFromFile(File imFile){
		
		org.eclipse.emf.common.util.URI emfURI = org.eclipse.emf.common.util.URI.createURI(imFile.toURI().toString());

		ResourceSet resourceSetIM = new ResourceSetImpl();
		// Force Package Registration
		@SuppressWarnings("unused")
		ImPackage imPackage = ImPackage.eINSTANCE;
		

		// load on demand = true
		Resource resourceIM = resourceSetIM.getResource(emfURI, true);

		// Get The intermediate model
		StpIntermediateModel imModel = (StpIntermediateModel) resourceIM.getContents().get(0);
		return imModel;
		
	}
	
	
	public void deploy(){
		
		logger.info("BPMGateway Deployment Helper::deploy() -> start deployment for process ["+process+"]");
		String imFileName = DeploymentUtils.getImFileLocation(process);
		logger.info("BPMGateway Deployment Helper::deploy() -> Retrieved IM File  ["+imFileName+"]");
		File imFile = new File(imFileName);
		
		if (!imFile.exists()){
			logger.info("BPMGateway Deployment Helper::deploy() -> IM File  ["+imFileName+"] not exist");
			throw new RuntimeException("Cannot locate IM File ["+imFileName+"] for process ["+process+"]");
		}
		
		Date currentDeployLastUpdatedTimeStamp =  ProcessEngine.normalizeDate(new Date(imFile.lastModified()));
		System.out.println("currentFileTimeStamp " + currentDeployLastUpdatedTimeStamp);
		if (!ProcessEngine.needToDeploy(process, currentDeployLastUpdatedTimeStamp)){
			logger.info("BPMGateway Deployment Helper::deploy() -> IM File  ["+imFileName+"] timestamp is not changed no deploy is needed ");
			return;
		}
		
		logger.info("deploy() -> IM File  ["+imFileName+"] is new or update start the deployment process ");
		
		StpIntermediateModel imModel = getImFromFile(imFile);
		logger.info("deploy() -> IM File  ["+imFileName+"] IM Model Retrieved ");
		
		
		Long deployedVersionOnJBPM = ProcessEngine.deployJdplFile(DeploymentUtils.getJPDLFileLocation(process));
		logger.info("JPDL Deployed With Version ["+deployedVersionOnJBPM+"] ");
		
	
		
		// Set the version deployed on JBPM to IM
		versionIM(imModel, deployedVersionOnJBPM);
		logger.info("Version Applied on IM ["+deployedVersionOnJBPM+"] ");
		
		publishIM(imModel, currentDeployLastUpdatedTimeStamp);
		logger.info("IMModel Published ["+deployedVersionOnJBPM+"] ");
		
		publishSchemas(imModel);
		
		
		
		
	}
	
	
	protected void versionIM(StpIntermediateModel im, Long version){
		String newVersion = String.valueOf(version);
		ImFactory stpModelfactory = ImFactory.eINSTANCE;
		for (org.eclipse.stp.im.Process p : im.getProcessCollection().getProcesses()){
			Property propertyVersion = p.getProperties().get(ImConstants.PROCESS_VERSION);
			if (propertyVersion != null){
				((BasicProperty)propertyVersion).setValue(newVersion);
			}else{
				propertyVersion  = stpModelfactory.createBasicProperty();
				propertyVersion.setKey(ImConstants.PROCESS_VERSION);
				((BasicProperty)propertyVersion).setValue(newVersion);
				p.getProperties().put(ImConstants.PROCESS_VERSION, propertyVersion);
			}
		}
	}

	
	protected void publishIM(StpIntermediateModel imModel, Date publishTimeStamp){
		
		Properties prefs = new Properties();
		try{
			File deploymentPreferenceFiles = new File(DeploymentUtils.getDeploymentPreferencesFileLocation());
			prefs.load(new FileInputStream(deploymentPreferenceFiles));
		}catch (Exception e) {
			throw new RuntimeException(e.getMessage(), e);
		}
		
		ISpagicJBPMCommand publishIntermediateModelCommand = new PublishIMCommand(imModel, prefs, publishTimeStamp);
		ProcessEngine.doWithinSpagic("publishIM()",null, publishIntermediateModelCommand, null);
	}
	
	protected void publishSchemas(StpIntermediateModel imModel){
		

		// Save all processes in process collection
		ProcessCollection processCollection = imModel.getProcessCollection();
		List<org.eclipse.stp.im.Process> processList = processCollection.getProcesses();
		for (org.eclipse.stp.im.Process process : processList) {
			
			// Save the steps
			List<Step> stepList = process.getSteps();
			for (Step step : stepList) {
				BasicProperty formProperty = (BasicProperty) step.getProperties().get(DeploymentUtils.FORM_SCHEMA_PROPERTY);
				if (formProperty != null) {
					String schemaValue = formProperty.getValue();
					
					if ((schemaValue != null) && (schemaValue.length() != 0)) {
						File schemaFile = new File(DeploymentUtils.getSchemaFileLocation(schemaValue));
						if (schemaFile.exists()) 
							publishSchema(schemaFile);
					}
				}
			}			
		}		
	}
	
	protected void publishSchema(final File f){
		ProcessEngine.doWithinSpagic("publishSchema()", null, 
				new ISpagicJBPMCommand() {
					
					@Override
					public Object execute(ProcessEngine processEngine, JbpmContext jbpmContext,
							Session aSession,
							ProcessInstanceExecutionContext instanceExecutionContext) {
						
						Document document = DeploymentUtils.fetchDocument(f);
						String idMetadata = DeploymentUtils.evalXPathAsString(document, "metadatadefinition/@id");

						SchemaMetadata smd = new SchemaMetadata();
						smd.setIdMetadata(idMetadata);
						smd.setDataSchema(document.asXML());
						
						aSession.saveOrUpdate(smd);
						
						// Force writing data on database
						aSession.flush();
						return null;
					}
				}, 
				null);
	}
	
}
