/**

    Copyright 2007, 2010 Engineering Ingegneria Informatica S.p.A.

    This file is part of Spagic.

    Spagic is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation; either version 3 of the License, or
    any later version.

    Spagic is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
    
**/
package org.spagic3.components.jdbc.config;

import java.util.List;
import java.util.Map;

import org.apache.commons.beanutils.ConvertUtils;
import org.apache.commons.codec.binary.Base64;
import org.dom4j.Document;
import org.dom4j.DocumentHelper;
import org.dom4j.Node;
import org.dom4j.xpath.DefaultXPath;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.spagic3.nmr.api.Message;

/**
 * Factory that initialize query parameter based on configuration and on
 * values retrived from input message.
 * 
 *
 */
public class JDBCParameterFactory {
	private static final Logger log = LoggerFactory.getLogger(JDBCParameterFactory.class);
	private static JDBCParameterFactory singleton;
	
	
	public static JDBCParameterFactory getInstance() {
		if (singleton == null) {
			singleton = new JDBCParameterFactory();
		}
		return singleton;
	}
	
	
	
	public Object initializeParameter(Message msg,
		QueryParameterConfig config) throws Exception {
		
		
		Object xmlValue = retrieveMessageInEnvelope(msg,
			config.getXpath());
		
		log.debug("Converting value: "+xmlValue+" to type: "+getClassFromConfig(config));
		Object value = ConvertUtils
			.convert((String)xmlValue, getClassFromConfig(config));
		
		if ( isBase64(retrieveParamType(config)) ) {
			return Base64.decodeBase64(value.toString().getBytes());
		}
		
		return value;
	}
	
    private Object retrieveMessageInEnvelope(Message in, String xpathString)
		throws Exception {

        Document doc = DocumentHelper.parseText((String)in.getBody());
        Map<String,String> namespaces = XmlNamespaceUtils.retrieveNameSpace(doc);
        if (xpathString.startsWith("node(")){
        	String nodeSelectionXPath = xpathString.substring(xpathString.indexOf("(")+1,xpathString.indexOf(")"));
        	Node n = doc.selectSingleNode(nodeSelectionXPath);
        	if (n == null){
        		log.warn("Node Selected by expression ["+nodeSelectionXPath+"] is NULL");
        		return "";
        	}
        	return n.asXML();
        }else{
        	DefaultXPath xPath = new DefaultXPath(xpathString); 
        	String nmEnvelope = null;
		
        	xPath.setNamespaceURIs(namespaces);
        	
        	// Check if the expression selects an element
        	Object selectedObj = xPath.selectObject(doc);
        	if (selectedObj instanceof List) {
				List objList = (List) selectedObj;
				if (objList.size() != 0) {
					// At least one object was selected
		        	nmEnvelope = xPath.valueOf(doc);
				} else {
	        		log.debug("No nodes were selected by expression ["+xpathString+"]");
				}
        	} else {
            	nmEnvelope = xPath.valueOf(doc);
        	}
        	
        	return nmEnvelope;
        }
        
	}

	private boolean isBase64(String paramType) {
		return QueryParameterConfig.BINARY_BASE64_TYPE
			.equals(paramType);
	}
	
	private Class getClassFromConfig(QueryParameterConfig config)
		throws ClassNotFoundException {
		
		String pType = retrieveParamType(config);
		if ( isBase64(pType) ) {
			
			return String.class;
		}
		return Class.forName(pType);
	}
	
	private String retrieveParamType(QueryParameterConfig config){
		
		String t = config.getParamType();
		
		if (t.indexOf("(") > 0 && t.indexOf(")") > 0)
			t = t.substring(t.indexOf("(")+1, t.indexOf(")"));
		return t;
	}
}
