package org.spagic3.connectionfactory;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.concurrent.ConcurrentHashMap;

import javax.jms.ConnectionFactory;

import org.osgi.framework.ServiceReference;
import org.osgi.service.component.ComponentContext;
import org.osgi.service.event.Event;
import org.osgi.service.event.EventAdmin;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.spagic3.constants.SpagicConstants;
import org.spagic3.core.SpagicNode;


public class ConnectionFactoryManager implements IConnectionFactoryManager {
	
	private EventAdmin ea = null;
	
	private static Logger logger = LoggerFactory.getLogger(ConnectionFactoryManager.class);
	private ConcurrentHashMap<String, ServiceReference> connectionFactories = new ConcurrentHashMap<String, ServiceReference>();
	private ComponentContext componentContext = null;
	private List<String> interestToNotification = new ArrayList<String>();
	
	protected void activate(ComponentContext componentContext) {
		logger.info(" ConnectionFactoryManager - ACTIVATED");
		this.componentContext = componentContext;
	}

	protected void deactivate(ComponentContext componentContext) {
		logger.info(" ConnectionFactoryManager - DEACTIVATED");
		
	}
	
	public void registerForNotification(String id){
			synchronized (this) {
				interestToNotification.add(id);
			}
	}
	
	public void unregisterForNotification(String id){
		synchronized (this) {
			interestToNotification.remove(id);
		}
}

	public void addConnectionFactory(ServiceReference ConnectionFactoryServiceReference) {
		
		String cfIdentifier = (String) ConnectionFactoryServiceReference
				.getProperty("id");
		
		logger.info(" ConnectionFactory ["+cfIdentifier+"] -- REGISTERED");
		
		connectionFactories.put(cfIdentifier, ConnectionFactoryServiceReference);
		notify(cfIdentifier, SpagicConstants._INTERNAL_EVENT_CF_DEPLOYED);
		notifyInterested(cfIdentifier, SpagicConstants._INTERNAL_EVENT_CF_DEPLOYED);

		
	}

	public void removeConnectionFactory(
			ServiceReference ConnectionFactoryServiceReference) {

		String dsIdentifier = (String) ConnectionFactoryServiceReference.getProperty("id");
		
		logger.info(" ConnectionFactory ["+dsIdentifier+"] -- UNREGISTERED");
		connectionFactories.remove(dsIdentifier);
		notify(dsIdentifier, SpagicConstants._INTERNAL_EVENT_CF_UNDEPLOYED);
		notifyInterested(dsIdentifier, SpagicConstants._INTERNAL_EVENT_CF_UNDEPLOYED);
	}

	@Override
	public ConnectionFactory getConnectionFactory(String ConnectionFactoryIdentifier) throws Exception {
		ServiceReference sr = connectionFactories.get(ConnectionFactoryIdentifier);
		
		if (sr == null){
			logger.error("ConnectionFactoryManager -> No ConnectionFactory registered with id ["+ConnectionFactoryIdentifier+"]");
			throw new Exception("ConnectionFactoryManager -> No ConnectionFactory registered with id ["+ConnectionFactoryIdentifier+"]");
		}
		ConnectionFactory cf = (ConnectionFactory)componentContext.locateService("connectionfactories", sr);
		return cf;		
	}	
	
	public void bind(EventAdmin ea){
		this.ea = ea;
	}

	public void unbind(EventAdmin ea){
		this.ea = null;
	}
	
	
	public void notify(String id, String eventType) {
		
		Map<String, Object> internalEventProperties = new HashMap<String, Object>();
		internalEventProperties.put(SpagicConstants._IS_INTERNAL_EVENT, true);
		internalEventProperties.put(SpagicConstants._INTERNAL_EVENT_TYPE, eventType);
		internalEventProperties.put(SpagicConstants._INTERNAL_EVENT_REFERRING_ID, id);
		Event ev = new Event(SpagicConstants.SPAGIC_GENERIC_TOPIC, internalEventProperties);
		if (this.ea != null){
			ea.postEvent(ev);
		}
	}
	
	public void notifyInterested(String id, String eventType) {
		synchronized (eventType) {
			for (String interested : interestToNotification){
				Map<String, Object> internalEventProperties = new HashMap<String, Object>();
				internalEventProperties.put(SpagicConstants._IS_INTERNAL_EVENT, true);
				internalEventProperties.put(SpagicConstants._INTERNAL_EVENT_TYPE, eventType);
				internalEventProperties.put(SpagicConstants._INTERNAL_EVENT_REFERRING_ID, id);
				Event ev = new Event(SpagicNode.topicForService(interested), internalEventProperties);
				if (this.ea != null){
					ea.postEvent(ev);
				}
			}
		}
		
	}
	

	@Override
	public Set<String> getRegisteredConnectionFactoriesIDs() {
		return connectionFactories.keySet();
	}

	
}
