package org.spagic3.connectors.http.adapters;

import java.io.BufferedInputStream;
import java.io.ByteArrayInputStream;
import java.io.FilterOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.io.PrintWriter;
import java.io.SequenceInputStream;
import java.io.StringWriter;
import java.util.ArrayList;
import java.util.Enumeration;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Properties;

import javax.activation.DataHandler;
import javax.mail.Header;
import javax.mail.MessagingException;
import javax.mail.Session;
import javax.mail.internet.ContentType;
import javax.mail.internet.MimeBodyPart;
import javax.mail.internet.MimeMessage;
import javax.mail.internet.MimeMultipart;
import javax.mail.util.ByteArrayDataSource;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.dom4j.Document;
import org.dom4j.DocumentException;
import org.dom4j.DocumentHelper;
import org.dom4j.Element;
import org.dom4j.io.SAXReader;
import org.dom4j.io.XMLWriter;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.spagic3.connectors.http.adapters.util.HTTPConnectorUtil;
import org.spagic3.constants.SpagicConstants;
import org.spagic3.core.ExchangeUtils;
import org.spagic3.nmr.api.Exchange;
import org.spagic3.nmr.api.Message;
import org.spagic3.nmr.api.Pattern;
import org.spagic3.soap.bindings.http.HttpConstants;
import org.spagic3.util.properties.PropertyConfigurator;

public class ExtendedPlainHTTPInputProtocolAdapter implements IHTTPInputProtocolAdapter {

	public static final String MULTIPART_CONTENT = "multipart/";
	private static final String prmPrefix = "HTTP-PRM-";
	
	public String getAdapterId() {
		return "ExtendedPlainHTTP";
	}
	
	
	@Override
	public Exchange createExchange(HttpServletRequest request, String mep,
			String sender, String target, PropertyConfigurator pc,
			Object binding) throws Exception {
		
		Logger logger = LoggerFactory.getLogger(ExtendedPlainHTTPInputProtocolAdapter.class);

		Exchange exchange = null;
		String bodyMessage = null;
		
		// Set the exchange corretly
		if (mep.equalsIgnoreCase(SpagicConstants.IN_OUT_MEP)){
			exchange = ExchangeUtils.createExchange(sender,target, Pattern.InOut);
        }else if (mep.equalsIgnoreCase(SpagicConstants.IN_ONLY_MEP)){
        	exchange = ExchangeUtils.createExchange(sender,target, Pattern.InOnly);
        }
		
		// Get the reference to the message
		Message in = exchange.getIn();
		
		// Get the method by which the service is invoked
		String reqMethod = request.getMethod();
		
		// Get the content-type of the http request
		String reqContentType = request.getContentType();
		if (reqContentType==null || reqContentType.equals("")) reqContentType = "application/octet-stream";
		
		// Get the uri of the http request
		String reqUri = request.getRequestURL().toString();
		
		// Map Object used for all parameters to put in Exchange header
		Map<String,Object> paramToHeader = new HashMap<String,Object>();     
		
		// Copy the generic info to the map for Exchange header
		paramToHeader.put(HttpConstants.REQUEST_URI, reqUri);
		paramToHeader.put(HttpConstants.CONTENT_TYPE, (reqContentType==null?"":reqContentType));
		paramToHeader.put(HttpConstants.REQUEST_METHOD, reqMethod);
		
		// Copy of the header HTTP parameters to the map for Exchange header
		for (Enumeration<?> e = request.getHeaderNames(); e.hasMoreElements();) {
            String name = (String) e.nextElement();
            String value = request.getHeader(name);
            paramToHeader.put(name, value);
        }
		
		// If the content-type is multipart, read the attachments
		if ((reqContentType != null)
				&& (reqContentType.toLowerCase().startsWith(MULTIPART_CONTENT))) {

			InputStream is = request.getInputStream();
			is = new SequenceInputStream(new ByteArrayInputStream(new byte[] {
					13, 10 }), is);
			Session session = Session.getDefaultInstance(new Properties());
			MimeMessage mime = new MimeMessage(session, is);
			mime.setHeader("Content-Type", reqContentType);

			if (reqContentType.toLowerCase().indexOf("related") == -1) {
				// Read the multipart and set the content of Exchange body
				bodyMessage = read(in, mime, paramToHeader);
			} else {
				// Read the multipart and set the content of Exchange body
				bodyMessage = read(in, mime);
			}
		} else {
			if (reqMethod.toLowerCase().equals(
					HttpConstants.METHOD_GET.toLowerCase())) {
				// Copy of the query-string HTTP parameters to the Exchange
				// header
				String queryString = request.getQueryString();
				if (queryString != null) {
					String[] params = queryString.split("&");
					for (String param : params) {
						String name = param.split("=")[0];
						String value = param.split("=")[1];
						paramToHeader.put(name, value);
					}
				}

				// Set the content of Exchange body
				bodyMessage = createBodyMessage(request);
			} else if (reqMethod.toLowerCase().equals(
					HttpConstants.METHOD_POST.toLowerCase())) {
				
				// If the content-type is application/x-www-form-urlencoded
				// the informations are passed as parameters
				// Else the informations are a message in the body 
				if (reqContentType.toLowerCase().indexOf(HTTPConnectorUtil.HTTP_HV_APPL_URL_ENC) != -1) 
				{
					// Copy of the POST or GET HTTP parameters to the Exchange
					// header
					Enumeration<String> enu = request.getParameterNames();
					int numParams = 0;
					String name = "";
					String value = "";
					while (enu.hasMoreElements()) {
						name = enu.nextElement();
						value = request.getParameter(name);
						paramToHeader.put(name, value);
						numParams++;
					}
					
					// Set a content of Exchange body whith no information
					bodyMessage = HTTPConnectorUtil.createXMLBody("");
				} 
				else 
				{
					// Set a content of Exchange body whith the xml
					bodyMessage = createBodyMessage(request);
				}

			}
		}

		// Set the Exchenage header
		in.setHeaders(paramToHeader);

		// Set the new Exchenage body
		in.setBody(bodyMessage);

		// Logging Header
		Map<String, Object> headers = in.getHeaders();
		for (Map.Entry<String, Object> e : headers.entrySet()) {
			logger.debug("Header " + e.getKey() + " ==> " + e.getValue());
		}

		// Logging Header
		Map<String, Object> attachs = in.getAttachments();
		for (Map.Entry<String, Object> e : attachs.entrySet()) {
			logger.debug("Attach " + e.getKey() + " ==> " + e.getValue());
		}
		logger.debug("Body " + in.getBody());

		// Set the Message input
		exchange.setIn(in);

		return exchange;
	}

	@Override
	public void sendAccepted(Exchange exchange, HttpServletRequest request,
			HttpServletResponse response, PropertyConfigurator pc, Object binding) {
		
		// Set the response status to ACCEPTED
		response.setStatus(HttpServletResponse.SC_ACCEPTED);
		
	}

	@Override
	public void sendError(Exchange exchange, Exception error,
			HttpServletRequest request, HttpServletResponse response,
			PropertyConfigurator pc, Object binding) throws Exception {
		
		// Set the response status to INTERNAL SERVER ERROR
		response.setStatus(HttpServletResponse.SC_INTERNAL_SERVER_ERROR);
        
		// Create the response with error message
		// (xml document with 'error' as root element)
		XMLWriter responseWriter = new XMLWriter(response.getWriter());
        Document doc = DocumentHelper.createDocument();
        Element root = doc.addElement("error");
        StringWriter sw = new StringWriter();
        PrintWriter pw = new PrintWriter(sw);
        error.printStackTrace(pw);
        pw.close();
        root.addText(sw.toString());
        responseWriter.write(doc);
        responseWriter.flush();
        responseWriter.close();
		
	}

	@Override
	public void sendFault(Exchange exchange, Message fault,
			HttpServletRequest request, HttpServletResponse response,
			PropertyConfigurator pc, Object binding) throws Exception {
		
		// Set the response status to BAD REQUEST
		response.setStatus(HttpServletResponse.SC_BAD_REQUEST);
		
		// Create the response
		XMLWriter responseWriter = new XMLWriter(response.getWriter());
		Document doc = DocumentHelper.parseText((String)fault.getBody());
		responseWriter.write(doc);
		responseWriter.flush();
		responseWriter.close();
		
	}

	@Override
	public void sendOut(Exchange exchange, Message out,
			HttpServletRequest request, HttpServletResponse response,
			PropertyConfigurator pc, Object binding) throws Exception {
	
		byte[] byteBuffer = null;
		
		// Set the response status to OK
		response.setStatus(HttpServletResponse.SC_OK);
		
		// Copy all Exchange header with prefix "HTTP-PRM-"
		// in HTTP Response header
		Map<String,Object> headers = out.getHeaders();
		for (Map.Entry<String, Object> e : headers.entrySet()) {
			// get key
			String aKey = e.getKey();
			// defensive test
			if (aKey!= null){
				// test if is a special param (begins with "HTTP-PRM-" or not)
				if (aKey.startsWith(prmPrefix)){
					// strip prefix
					String tmpPrmName = aKey.substring(prmPrefix.length());
					// put inside map
					response.addHeader(tmpPrmName, (String)e.getValue());
				}
			}
		}
		
		//Copy the Exchange body to the Response Body
		if (out.getBody() != null) {

			//verify if bodyXML doesn't contain plainText inside CDATA
    		if ( ((String)out.getBody()).indexOf(HTTPConnectorUtil._HTTP_BODY) == -1 ){
    			byteBuffer = ((String)out.getBody()).getBytes(HTTPConnectorUtil._UTF8_);
    		} else {
    			//parse body
    			Document aDocument = DocumentHelper.parseText( new String( (String)out.getBody()));
    			//get CDATA
    			String tmpCDATA = new String(aDocument.selectSingleNode("//HTTP-BODY").getText());
    			byteBuffer = tmpCDATA.getBytes(HTTPConnectorUtil._UTF8_); 
    		}
		} 
		
		OutputStream baos = response.getOutputStream();
		try{
			// Create the response
			if ((out.getAttachments()!= null && out.getAttachments().size()>0)){
				writeMultipartAttachments(out, baos, byteBuffer, response);
				//httpEx.setRequestHeader(HttpHeaders.CONTENT_TYPE, (String)msg.getHeader(HttpConstants.CONTENT_TYPE));
				//httpEx.setRequestContent(new ByteArrayBuffer(baos.toByteArray()));
			} else 	{
				baos.write(byteBuffer);
			}
		} finally {
			baos.flush();
			baos.close();
		}

		
	}
	
	@SuppressWarnings("unchecked")
    public static void writeMultipartAttachments(Message message, OutputStream out, byte[] data, HttpServletResponse response) throws MessagingException, IOException {
        MimeMultipart parts = new MimeMultipart(HTTPConnectorUtil.HTTP_HV_RELATED);
        Session session = Session.getDefaultInstance(new Properties(), null);
        MimeMessage mime = new MimeMessage(session);
        // Add soap part
        MimeBodyPart soapPart = new MimeBodyPart();
        soapPart.setContentID("<body-part>");
        soapPart.setDataHandler(new DataHandler(new ByteArrayDataSource(data, HTTPConnectorUtil.HTTP_HV_TEXT_XML)));
        soapPart.setHeader(HTTPConnectorUtil.HTTP_CONTENT_TRANSFER_ENC, HTTPConnectorUtil.HTTP_HV_ENC_BIN);
        soapPart.setHeader(HTTPConnectorUtil.HTTP_CONTENT_TYPE, HTTPConnectorUtil.HTTP_HV_TEXT_XML_UTF8);
        parts.addBodyPart(soapPart);
        // Add attachments
        for (Iterator itr = message.getAttachments().entrySet().iterator(); itr.hasNext();) {
            Map.Entry entry = (Map.Entry) itr.next();
            String id = (String) entry.getKey();
            DataHandler dh = (DataHandler) entry.getValue();
            MimeBodyPart part = new MimeBodyPart();
            part.setDataHandler(dh);
            part.setContentID("<" + id + ">");
            part.setHeader(HTTPConnectorUtil.HTTP_CONTENT_TRANSFER_ENC, HTTPConnectorUtil.HTTP_HV_ENC_BIN);
            part.setHeader(HTTPConnectorUtil.HTTP_CONTENT_TYPE, HTTPConnectorUtil.HTTP_HV_APPL_OCTET);
            parts.addBodyPart(part);
        }
        mime.setContent(parts);
        mime.setHeader(HTTPConnectorUtil.HTTP_CONTENT_TYPE, parts.getContentType());
//        message.setHeader(HTTPConnectorUtil.HTTP_CONTENT_TYPE, parts.getContentType());
        response.setContentType(parts.getContentType());
        
        // We do not want headers, so 
        //  * retrieve all headers
        //  * skip first 2 bytes (CRLF)
        mime.saveChanges();
        Enumeration<Header> headersEnum = mime.getAllHeaders();
        List<String> headersList = new ArrayList<String>();
        while (headersEnum.hasMoreElements()) {
        	String tmp = headersEnum.nextElement().getName();
            headersList.add(tmp);
        }
        String[] headers = headersList.toArray(new String[0]);
        // Skip first 2 bytes
        OutputStream os = new FilterOutputStream(out) {
            private int nb = 0;
            public void write(int b) throws IOException {
                if (++nb > 2) {
                    super.write(b);
                }
            }
        };
        // Write
        mime.writeTo(os, headers);
    }
	
	
	private String read(Message message, MimeMessage mime, Map<String,Object> paramToHeader) throws MessagingException, IOException {
		
		String bodyMessage = "";
		
		// Content must be ah InputStream
		Object content = mime.getInputStream(); // You could also
		if (content instanceof InputStream == false) {
			throw new UnsupportedOperationException(
					"Expected a InputStream object but found a "
							+ content.getClass());
		}

		// Convert the InputStream to MimeMultipart
		InputStream inputStream = (InputStream) content;
		ByteArrayDataSource inboundDataSource = new ByteArrayDataSource(
				inputStream, mime.getContentType());
		MimeMultipart multipart = new MimeMultipart(inboundDataSource);

		// Analyze evry single part of multipart
		for (int i = 0; i < multipart.getCount(); i++) {
			MimeBodyPart part = (MimeBodyPart) multipart.getBodyPart(i);

			Header tmp = null;
			String name = "";
			String value = "";
			Enumeration<Header> enu1 = part.getAllHeaders();
			String partName = "";
			while (enu1.hasMoreElements()) {
				tmp = (Header) enu1.nextElement();
				name = tmp.getName();
				value = tmp.getValue();

				// Get the name of parameter
				if (name.toLowerCase().equals("content-disposition")) {
					int idx1 = value.indexOf("name=");
					if (idx1 != -1) {
						String subStr = value.substring(idx1);
						int idx2 = subStr.indexOf(";");
						if (idx2 == -1)
						{
							// char ';' not found
							partName = subStr.substring(5).trim();
						}
						else
						{
							// char ';' found
							partName = subStr.substring(5, idx2).trim();
						}
						// delete first and final char '"'
						if (partName.startsWith("\"")) partName = partName.substring(1);
						if (partName.endsWith("\"")) partName = partName.substring(0,(partName.length()-1));
						
						
//						String subStr = value.substring(idx1 + 6);
//						int idx2 = subStr.indexOf("\"");
//						if (idx2 != -1)
//							partName = subStr.substring(0, idx2);
					}
				}

			}

			// if filename is not empty, this is a file
			// You add it as Attachment
			if (part.getFileName() != null) {
				message.addAttachment(partName, part.getDataHandler());
			}
			// if filename is empty, this is a parameter or the body message
			else {
				InputStream isPart = part.getInputStream();
				String valuePart = HTTPConnectorUtil.convertStreamToString(isPart);

				// if part has no name, it is the body message
				// otherwise it is a parameter
				if (partName.equals("")) {
					InputStream js = new ByteArrayInputStream(valuePart.getBytes());
					if (HTTPConnectorUtil.isXML(js)){
						bodyMessage = valuePart;
					} else {
						// The part value is not a xml string
						bodyMessage = HTTPConnectorUtil.createXMLBody(valuePart);
					}
					if (js!=null) js.close();
				} else {
					// Add the parameter to Header
					paramToHeader.put(partName, valuePart);
				}
			}
		}

		// if bodyMessage is empty
		// set a default xml
		if (bodyMessage.equals(""))
			bodyMessage = HTTPConnectorUtil.createXMLBody("");

		return bodyMessage;
	}

    public String read(Message message, MimeMessage mime) throws MessagingException, IOException {
        
    	final Object content = mime.getContent();
		String bodyMessage = "";

        
        if (content instanceof MimeMultipart == false) {
            throw new UnsupportedOperationException("Expected a javax.mail.internet.MimeMultipart object but found a " + content.getClass());
        }
        MimeMultipart multipart = (MimeMultipart) content;
        ContentType type = new ContentType(mime.getContentType());
        String contentId = type.getParameter("start");
        
        // Get request
        MimeBodyPart contentPart = null;
        if (contentId != null) {
            contentPart = (MimeBodyPart) multipart.getBodyPart(contentId);
        } else {
            for (int i = 0; i < multipart.getCount(); i++) {
              MimeBodyPart contentPart2 = (MimeBodyPart) multipart.getBodyPart(i);
              String contentType = contentPart2.getContentType();
              
              if (contentType.indexOf("xml") >= 0) {
                contentPart = contentPart2;
                break;
              }
            }
        }
        
        // Set new content
        InputStream is = contentPart.getInputStream();
        bodyMessage = (is==null?"":HTTPConnectorUtil.convertStreamToString(is));
        InputStream js = new ByteArrayInputStream(bodyMessage.getBytes());
        if (!HTTPConnectorUtil.isXML(js)){
			// The part value is not a xml string
			bodyMessage = HTTPConnectorUtil.createXMLBody(bodyMessage);
		}
        if (js!=null) js.close();
        
        
        // Get attachments
        for (int i = 0; i < multipart.getCount(); i++) {
            MimeBodyPart part = (MimeBodyPart) multipart.getBodyPart(i);
            if (part != contentPart) {
                String id = part.getContentID();
                if (id == null) {
                    id = "Part" + i;
                } else if (id.startsWith("<")) {
                    id = id.substring(1, id.length() - 1);
                }
                message.addAttachment(id, part.getDataHandler());
            }
        }
        
        return bodyMessage;
    }
	
	
	
	private String createBodyMessage(HttpServletRequest request) throws IOException
	{		
		String res = "";

		InputStream is = request.getInputStream();
		is = new BufferedInputStream(is);
		SAXReader reader = new SAXReader();
		try {
			Document doc = reader.read(is);
			res = doc.asXML();
		} catch (DocumentException de) {
			// Convert the body InputStream to String
			String bodyString = HTTPConnectorUtil.convertStreamToString(is);

			// Build new xml
			res = HTTPConnectorUtil.createXMLBody(bodyString);
		}

		return res;
	}
		
}
