package org.spagic3.connectors.http.adapters;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.FilterOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.io.SequenceInputStream;
import java.io.UnsupportedEncodingException;
import java.net.URI;
import java.net.URLEncoder;
import java.util.ArrayList;
import java.util.Enumeration;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Properties;

import javax.activation.DataHandler;
import javax.mail.Header;
import javax.mail.MessagingException;
import javax.mail.Session;
import javax.mail.internet.ContentType;
import javax.mail.internet.MimeBodyPart;
import javax.mail.internet.MimeMessage;
import javax.mail.internet.MimeMultipart;
import javax.mail.util.ByteArrayDataSource;

import org.dom4j.Document;
import org.dom4j.DocumentHelper;
import org.mortbay.io.ByteArrayBuffer;
import org.mortbay.jetty.HttpFields;
import org.mortbay.jetty.HttpHeaders;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.spagic3.connectors.http.adapters.util.HTTPConnectorUtil;
import org.spagic3.nmr.api.Exchange;
import org.spagic3.nmr.api.Message;
import org.spagic3.nmr.api.Pattern;
import org.spagic3.nmr.api.Status;
import org.spagic3.soap.bindings.http.HttpConstants;
import org.spagic3.util.properties.PropertyConfigurator;

/**
 * Default marshaler used for non-soap provider endpoints.

 * @since 3.2
 */
public class ExtendedPlainHTTPOutputProtocolAdapter implements IHTTPOutputProtocolAdapter {

	protected Logger logger = LoggerFactory.getLogger(ExtendedPlainHTTPOutputProtocolAdapter.class);
	
	private static final String prmPrefix = "HTTP-PRM-";	
	private static final String _MULTIPART_RELATED = "multipart/related";	
	private static final String _MULTIPART_FORM = "multipart/form-data";
	private static final String _JBI = "JBI";
	
	private static final String _QUERYSTRING ="QueryString";
	private static final String _HTTPHEADER ="HTTPHeader";
	public static final String MULTIPART_PREFIX = "multipart/";
		
//	private String locationURI;
//    private String method = HttpConstants.METHOD_POST;
//    private String contentType = HTTPConnectorUtil.HTTP_HV_TEXT_XML;
//    private String params = _HTTPHEADER;
//    private String errorPolicy = _JBI;
//    private String multipartPolicy = _MULTIPART_RELATED;
    
    public String getAdapterId() {
		return "ExtendedPlainHTTP";
	}
	
//	@Override
//	public void configure(PropertyConfigurator propertyConfigurator) {
//		this.contentType = propertyConfigurator.getString(HTTPConnectorUtil.HTTPPLAIN_CFG_CONTENT);
//		this.method = propertyConfigurator.getString(HTTPConnectorUtil.HTTPPLAIN_CFG_METHOD);
//		this.params = propertyConfigurator.getString(HTTPConnectorUtil.HTTPPLAIN_CFG_PARAMS);
//		this.errorPolicy = propertyConfigurator.getString(HTTPConnectorUtil.HTTPPLAIN_CFG_ERROR_POLICY, "FLOW");
//		this.multipartPolicy = propertyConfigurator.getString(HTTPConnectorUtil.HTTPPLAIN_CFG_MULTIPART_POLICY, "multipart/related");		
//		this.locationURI = propertyConfigurator.getString("locationURI");
//	}
   
//    public String getLocationURI() {
//        return locationURI;
//    }
//
//    public void setLocationURI(String locationUri) {
//        this.locationURI = locationUri;
//    }
//
// 
//    public String getMethod() {
//        return method;
//    }
//
//    public void setMethod(String method) {
//        this.method = method;
//    }
//
//    
//    public String getContentType() {
//        return contentType;
//    }
//
//    public void setContentType(String contentType) {
//        this.contentType = contentType;
//    }
//    
//
//    public String getParams() {
//		return params;
//	}
//
//	public void setParams(String params) {
//		this.params = params;
//	}
//
//    public String getErrorPolicy() {
//		return errorPolicy;
//	}
//
//	public void setErrorPolicy(String errorPolicy) {
//		this.errorPolicy = errorPolicy;
//	}
//
//
//	public void setMultipartPolicy(String multipartPolicy) {
//		this.multipartPolicy = multipartPolicy;
//	}

	protected String getLocationUri(Exchange exchange, Message inMsg, PropertyConfigurator pc)  {
       if (inMsg.getHeader("locationURI") != null){
    	   return (String)inMsg.getHeader("locationURI");
       }else{
    	   return pc.getString("locationURI");
       }
       
    }

    protected String getMethod(Exchange exchange, Message inMsg, PropertyConfigurator pc)  {
    	if (inMsg.getHeader(HTTPConnectorUtil.HTTPPLAIN_CFG_METHOD) != null){
     	   return (String)inMsg.getHeader(HTTPConnectorUtil.HTTPPLAIN_CFG_METHOD);
        }else{
     	   return pc.getString(HTTPConnectorUtil.HTTPPLAIN_CFG_METHOD);
        }
    }

    protected String getContentType(Exchange exchange, Message inMsg, PropertyConfigurator pc) {
    	if (inMsg.getHeader(HTTPConnectorUtil.HTTPPLAIN_CFG_CONTENT) != null){
      	   return (String)inMsg.getHeader(HTTPConnectorUtil.HTTPPLAIN_CFG_CONTENT);
         }else{
      	   return pc.getString(HTTPConnectorUtil.HTTPPLAIN_CFG_CONTENT);
         }
    }

    protected String getParams(Exchange exchange, Message inMsg, PropertyConfigurator pc) {
    	if (inMsg.getHeader(HTTPConnectorUtil.HTTPPLAIN_CFG_PARAMS) != null){
      	   return (String)inMsg.getHeader(HTTPConnectorUtil.HTTPPLAIN_CFG_PARAMS);
         }else{
      	   return pc.getString(HTTPConnectorUtil.HTTPPLAIN_CFG_PARAMS);
         }
    }

    protected String getMultipartPolicy(Exchange exchange, Message inMsg, PropertyConfigurator pc) {
    	if (inMsg.getHeader(HTTPConnectorUtil.HTTPPLAIN_CFG_MULTIPART_POLICY) != null){
      	   return (String)inMsg.getHeader(HTTPConnectorUtil.HTTPPLAIN_CFG_MULTIPART_POLICY);
         }else{
      	   return pc.getString(HTTPConnectorUtil.HTTPPLAIN_CFG_MULTIPART_POLICY, "multipart/related");
         }
    }

	public String getErrorPolicy(PropertyConfigurator pc) {
		return pc.getString(HTTPConnectorUtil.HTTPPLAIN_CFG_ERROR_POLICY, "FLOW");
	}

	public void fillJettyExchange(Exchange exchange,
			SpagicJettyHTTPExchange httpExchange, PropertyConfigurator pc,
			Object binding) {
    	
    	byte[] byteBuffer = null;
    	boolean isAttachmentPresent = false;
        
    	try{
        	Message inMsg = exchange.getIn(false);
        	

        	if ((inMsg.getAttachments()!= null && inMsg.getAttachments().size()>0)) 
        		isAttachmentPresent = true;

        	// get Parameters from  message or configs
			String msgLocationUri = getLocationUri(exchange, inMsg, pc);
			String msgMethod = getMethod(exchange, inMsg, pc);
			String msgContentType = getContentType(exchange, inMsg, pc);
			String msgParams = getParams(exchange, inMsg, pc);
			String msgMultipartPolicy = getMultipartPolicy(exchange, inMsg, pc);

        	httpExchange.setURL(msgLocationUri);

        	// Temporary fix for bug in jetty-client 6.1.5
        	// http://fisheye.codehaus.org/browse/jetty-contrib/jetty/trunk/contrib/client/src/main/java/org/mortbay/jetty/client/HttpConnection.java?r1=374&r2=378
        	httpExchange.addRequestHeader(HttpHeaders.HOST_BUFFER, new ByteArrayBuffer(new URI(msgLocationUri).getHost()));
        	httpExchange.setMethod(msgMethod);
        	
        	// get headers from inMsg
        	Map<String, Object> headers = inMsg.getHeaders();
        		
			// create new temporary map
			Map <String, String> httpParameters = null;
        		
    		// ********************** headers from NMR ********************************
        	// if headers are present...
        	if (headers != null) {
        		httpParameters = manageHeadersFromNMR(headers, httpExchange);
    		} // *********** end of HEADERS from NMR *****
        		
        		
    		// **************** parameters from NMR to httprequest *****************
    		if (httpParameters!=null){
    			byteBuffer = setParametersFromNMR (httpParameters, httpExchange, msgLocationUri, 
						   							msgParams, msgMultipartPolicy, msgMethod, 
						   							isAttachmentPresent);
    		} // ******** END OF parameters from NMR to httprequest 

        	
    		// ************* NMR BODY to httprequest *********************
    		// strategy : only if byteBuffer==null (if special param are present --> ignore body)
    		if (byteBuffer == null && inMsg.getBody() != null){

    			//verify if bodyXML doesn't contain plainText inside CDATA
        		if ( ((String)inMsg.getBody()).indexOf(HTTPConnectorUtil._HTTP_BODY) == -1 ){
        			byteBuffer = ((String)inMsg.getBody()).getBytes(HTTPConnectorUtil._UTF8_);
        		} else {
        			//parse body
        			Document aDocument = DocumentHelper.parseText( new String( (String)inMsg.getBody()));
        			//get CDATA
        			String tmpCDATA = new String(aDocument.selectSingleNode("//HTTP-BODY").getText());
        			byteBuffer = tmpCDATA.getBytes(HTTPConnectorUtil._UTF8_); 
        		}
    		} // *** END OF BODY from NMR to httprequest *****
        
    		// case 3
    		if ( msgParams.equals(_QUERYSTRING) && ( (msgMethod.equals(HttpConstants.METHOD_PUT)) || (msgMethod.equals(HttpConstants.METHOD_DELETE)) ) ){
    			msgContentType = HTTPConnectorUtil.HTTP_HV_APPL_URL_ENC;
    		} 		
    		// case 4
    		if ( (msgParams.equals(_QUERYSTRING)) && (msgMethod.equals(HttpConstants.METHOD_POST)) && (!isAttachmentPresent) ){
    			// get querystring from httpHeaders and prepare it as bytebuffer
    			msgContentType = HTTPConnectorUtil.HTTP_HV_APPL_URL_ENC;
    		} 		
    		
    		// ********************** postage & packing --> manage attachment if presents ******************
    		prepareHttpJettyExchange (httpParameters, httpExchange, msgParams, msgMultipartPolicy, 
    									msgMethod, msgContentType, inMsg, byteBuffer);
        }catch (Exception e) {
			throw new RuntimeException(e);
		}
    }

	public void handleResponse(Exchange exchange,
			SpagicJettyHTTPExchange httpExchange, PropertyConfigurator pc,
			Object binding) {
    	
    	// get HTTPbody
    	String xmlBody = null;
    	HttpFields httpHeaders = null;
    	Map <String, Object> headerFromMultipartFormData = null;
    	
    	try{
    		// get answer status
    		int responseStatus = httpExchange.getResponseStatus();

    		// failed invocation && error policy = JBI
    		if (	responseStatus != org.mortbay.jetty.HttpStatus.ORDINAL_200_OK && 
    				responseStatus != org.mortbay.jetty.HttpStatus.ORDINAL_202_Accepted &&
    				getErrorPolicy(pc).equals(_JBI) ) 
    		{

    			// In Only
    			if (exchange.getPattern() == Pattern.InOnly){
    				exchange.setError(new Exception("Invalid status response: " + responseStatus));
	    		}

    			// InOut
    			if (exchange.getPattern() == Pattern.InOut){
    				Message fault = exchange.getFault(true);
    				fault.setBody(new String(httpExchange.getResponse()));
    				exchange.setFault(fault);
	    		}
    			
       		// successful invocation or error + FLOW error management policy
    		} else  {

    			// manage HTTP header & content only if Pattern.InOut
    			if (exchange.getPattern() == Pattern.InOut) {
					// get header
					httpHeaders = httpExchange.getResponseFields();
	    			String contentType = httpHeaders.getStringField(HTTPConnectorUtil.HTTP_CONTENT_TYPE);
					
	    			// check if multipart or not
	    	        if (contentType != null && contentType.toLowerCase().startsWith(MULTIPART_PREFIX)) {
	    	        	
	    	        	// is multipart
	    	        	InputStream is = new ByteArrayInputStream(httpExchange.getResponse());
	                    Session session = Session.getDefaultInstance(new Properties());
	                    is = new SequenceInputStream(new ByteArrayInputStream(new byte[] { 13, 10 }), is);
	                    MimeMessage mime = new MimeMessage(session, is);
	                    mime.setHeader(HTTPConnectorUtil.HTTP_CONTENT_TYPE, contentType);
	                    
	                    // case for different types of multipart
	                    String multipartType = contentType.substring(MULTIPART_PREFIX.length());
	                    
	                    if (multipartType.startsWith(HTTPConnectorUtil.HTTP_HV_FORMDATA)){
	                    	headerFromMultipartFormData = new HashMap<String, Object>();
	                    	xmlBody = readMultipartFormData(exchange.getOut(), mime, headerFromMultipartFormData);
	                    } else if (multipartType.startsWith("related")){
	                    	xmlBody = readMultipartRelated(exchange.getOut(), mime);
	                    } else {
	                    	
	                    }
	    	        	
	    	        } else{
	    	        	// ISN'T multipart
	    	        	byte[] tmpArray = httpExchange.getResponse();
	    	        	if (tmpArray==null || tmpArray.length==0) 
	    	        		xmlBody = "";
	    	        	else
	    	        		xmlBody = new String(tmpArray);
	    	        }
    			}
    	        
    	        // parse xml only if body expected due to Pattern.InOut
    	        if (exchange.getPattern() == Pattern.InOut){
    	        	// 	create consumable inputStream
    	        	ByteArrayInputStream bais = new ByteArrayInputStream(xmlBody.getBytes(HTTPConnectorUtil._UTF8_));
				    			
    	        	// 	successful invocation
	    			if (	responseStatus == org.mortbay.jetty.HttpStatus.ORDINAL_200_OK || 
	        				responseStatus == org.mortbay.jetty.HttpStatus.ORDINAL_202_Accepted) {
	
	    				// verify if httpbody is XML
	    				if (!HTTPConnectorUtil.isXML(bais)){
	        				// encapsulate body into XML
	    					xmlBody = HTTPConnectorUtil.createXMLBody(xmlBody);
	    				}
	    				// close buffer
	    				bais.close();
	    				
	    			// failed invocation && error policy = FLOW
	    			} else {
	    				// encapsulate body into XML ERROR
	    				xmlBody = HTTPConnectorUtil.createXMLError(responseStatus, xmlBody); 
	    			}
    	        }

    			// set exchange
    			if (exchange.getPattern() == Pattern.InOut){
    				Message out = exchange.getOut();
    				
    				// set body
    				out.setBody(xmlBody);

    				// manage header & attachment only if status = (200, 202)
    				if (	responseStatus == org.mortbay.jetty.HttpStatus.ORDINAL_200_OK || 
    	    				responseStatus == org.mortbay.jetty.HttpStatus.ORDINAL_202_Accepted) {
    					
    					// set header
    					httpHeaders = httpExchange.getResponseFields();
    					// get names
    					Enumeration<String> e = httpHeaders.getFieldNames();
    					// iterate all names
    					while (e.hasMoreElements()){
    						String aKey = e.nextElement();
    						if (aKey != null) {
    							out.setHeader(aKey, httpHeaders.getStringField(aKey));
    						}
    					}

    					// iterate all fields from form-data
    					if (headerFromMultipartFormData!=null && headerFromMultipartFormData.size()>0){
    						for (Map.Entry<String, Object> e1 : headerFromMultipartFormData.entrySet()) {
    							String aKey = e1.getKey();
    							if (aKey != null) {
        							out.setHeader(aKey, e1.getValue());
        						}
    						}
    					}
    					
    				}
    				
    				exchange.setOut(out);
    			} else {
    				exchange.setStatus(Status.Done);
    			}
    		}
    		
    	} catch (Exception e) {
			throw new RuntimeException(e);
		}
    }

	public void handleException(Exchange exchange,
			SpagicJettyHTTPExchange httpExchange, Throwable ex,
			PropertyConfigurator pc, Object binding) {
        exchange.setError((Exception)ex);
    }

    
    @SuppressWarnings("unchecked")
    public static void writeMultipartAttachments(Message message, OutputStream out, byte[] data) throws MessagingException, IOException {
        MimeMultipart parts = new MimeMultipart(HTTPConnectorUtil.HTTP_HV_RELATED);
        Session session = Session.getDefaultInstance(new Properties(), null);
        MimeMessage mime = new MimeMessage(session);
        // Add soap part
        MimeBodyPart soapPart = new MimeBodyPart();
        soapPart.setContentID("<body-part>");
        soapPart.setDataHandler(new DataHandler(new ByteArrayDataSource(data, HTTPConnectorUtil.HTTP_HV_TEXT_XML)));
        soapPart.setHeader(HTTPConnectorUtil.HTTP_CONTENT_TRANSFER_ENC, HTTPConnectorUtil.HTTP_HV_ENC_BIN);
        soapPart.setHeader(HTTPConnectorUtil.HTTP_CONTENT_TYPE, HTTPConnectorUtil.HTTP_HV_TEXT_XML_UTF8);
        parts.addBodyPart(soapPart);
        // Add attachments
        for (Iterator itr = message.getAttachments().entrySet().iterator(); itr.hasNext();) {
            Map.Entry entry = (Map.Entry) itr.next();
            String id = (String) entry.getKey();
            DataHandler dh = (DataHandler) entry.getValue();
            MimeBodyPart part = new MimeBodyPart();
            part.setDataHandler(dh);
            part.setContentID("<" + id + ">");
            part.setHeader(HTTPConnectorUtil.HTTP_CONTENT_TRANSFER_ENC, HTTPConnectorUtil.HTTP_HV_ENC_BIN);
            part.setHeader(HTTPConnectorUtil.HTTP_CONTENT_TYPE, HTTPConnectorUtil.HTTP_HV_APPL_OCTET);
            parts.addBodyPart(part);
        }
        mime.setContent(parts);
        mime.setHeader(HTTPConnectorUtil.HTTP_CONTENT_TYPE, parts.getContentType());
        message.setHeader(HTTPConnectorUtil.HTTP_CONTENT_TYPE, parts.getContentType());
        // We do not want headers, so 
        //  * retrieve all headers
        //  * skip first 2 bytes (CRLF)
        mime.saveChanges();
        Enumeration<Header> headersEnum = mime.getAllHeaders();
        List<String> headersList = new ArrayList<String>();
        while (headersEnum.hasMoreElements()) {
        	String tmp = headersEnum.nextElement().getName();
            headersList.add(tmp);
        }
        String[] headers = headersList.toArray(new String[0]);
        // Skip first 2 bytes
        OutputStream os = new FilterOutputStream(out) {
            private int nb = 0;
            public void write(int b) throws IOException {
                if (++nb > 2) {
                    super.write(b);
                }
            }
        };
        // Write
        mime.writeTo(os, headers);
    }

    @SuppressWarnings("unchecked")
    public static void writeMultipartForm(Message message, Map<String, String> aMap, OutputStream out, byte[] data) throws MessagingException, IOException {
        MimeMultipart parts = new MimeMultipart(HTTPConnectorUtil.HTTP_HV_FORMDATA);
        Session session = Session.getDefaultInstance(new Properties(), null);
        MimeMessage mime = new MimeMessage(session);
        
        if (data!=null && data.length>0)
        {
	        // Add soap part
	        MimeBodyPart soapPart = new MimeBodyPart();
	        
	        soapPart.setHeader(HTTPConnectorUtil.HTTP_CONTENT_DISPOSITION, HTTPConnectorUtil.HTTP_HV_FORMDATA);
	        soapPart.setDataHandler(new DataHandler(new ByteArrayDataSource(data, HTTPConnectorUtil.HTTP_HV_TEXT_XML)));
	        soapPart.setHeader(HTTPConnectorUtil.HTTP_CONTENT_TRANSFER_ENC, HTTPConnectorUtil.HTTP_HV_ENC_8BIT);
	        soapPart.setHeader(HTTPConnectorUtil.HTTP_CONTENT_TYPE, HTTPConnectorUtil.HTTP_HV_TEXT_XML_UTF8);
	        parts.addBodyPart(soapPart);
        }

        // add parameters
        if(aMap!= null && aMap.size()>0){
        	// iterate all params
        	for (Map.Entry<String, String> e : aMap.entrySet()) {
                String id = (String) e.getKey();
                DataHandler dh = new DataHandler(new ByteArrayDataSource(e.getValue().getBytes(HTTPConnectorUtil._UTF8_), HTTPConnectorUtil.HTTP_HV_TEXT_XML));
                MimeBodyPart part = new MimeBodyPart();
                part.setDataHandler(dh);
                part.setHeader(HTTPConnectorUtil.HTTP_CONTENT_DISPOSITION, "form-data; name=\""+ id +"\"");
	            part.setHeader(HTTPConnectorUtil.HTTP_CONTENT_TYPE, HTTPConnectorUtil.HTTP_HV_TEXT_XML_UTF8);
                part.setHeader(HTTPConnectorUtil.HTTP_CONTENT_TRANSFER_ENC, HTTPConnectorUtil.HTTP_HV_ENC_8BIT);
                parts.addBodyPart(part);
        	}
        }
        
        // Add attachments
        if (message.getAttachments()!=null && message.getAttachments().size()>0){
	        for (Iterator itr = message.getAttachments().entrySet().iterator(); itr.hasNext();) {
	            Map.Entry entry = (Map.Entry) itr.next();
	            String id = (String) entry.getKey();
	            DataHandler dh = (DataHandler) entry.getValue();
	            MimeBodyPart part = new MimeBodyPart();
	            part.setDataHandler(dh);
	            part.setHeader(HTTPConnectorUtil.HTTP_CONTENT_DISPOSITION, "form-data; name=\""+ id +"\"; filename=\""+ id +"\"");
	            //String headerAttachContentType = (String)message.getHeader(id.replaceAll("[.]", "_")+HTTPConnectorUtil.HTTP_ATTACH_CONTENT_TYPE_HEADER_SUFFIX);
	            String headerAttachContentType = (String)message.getHeader(id+HTTPConnectorUtil.HTTP_ATTACH_CONTENT_TYPE_HEADER_SUFFIX);
	            if ((headerAttachContentType !=null) && (!headerAttachContentType.equals("")))
	            {
	            	part.setHeader(HTTPConnectorUtil.HTTP_CONTENT_TYPE, headerAttachContentType);
	            }
	            //part.setHeader(HTTPConnectorUtil.HTTP_CONTENT_TYPE, HTTPConnectorUtil.HTTP_HV_APPL_OCTET);
	            part.setHeader(HTTPConnectorUtil.HTTP_CONTENT_TRANSFER_ENC, HTTPConnectorUtil.HTTP_HV_ENC_BIN);
	            parts.addBodyPart(part);
	        }
        }

        mime.setContent(parts);
        String contentType = parts.getContentType().replaceAll("(\r\n\t)+","");
        mime.setHeader(HTTPConnectorUtil.HTTP_CONTENT_TYPE, contentType);
//        mime.addHeader(HttpConstants.HTTP_CONTENT_TRANSFER_ENC, HttpConstants.HTTP_HV_ENC_BIN);
        message.setHeader(HTTPConnectorUtil.HTTP_CONTENT_TYPE, contentType);
        // We do not want headers, so 
        //  * retrieve all headers
        //  * skip first 2 bytes (CRLF)
        mime.saveChanges();
        Enumeration<Header> headersEnum = mime.getAllHeaders();
        List<String> headersList = new ArrayList<String>();
        while (headersEnum.hasMoreElements()) {
        	String tmp = headersEnum.nextElement().getName();
            headersList.add(tmp);
        }
        String[] headers = headersList.toArray(new String[0]);
        // Skip first 2 bytes
        OutputStream os = new FilterOutputStream(out) {
            private int nb = 0;
            public void write(int b) throws IOException {
                if (++nb > 2) {
                    super.write(b);
                }
            }
        };
        
        // Write
        mime.writeTo(os, headers);
    }
 
    public static String getEncodedQueryString(Map <String, String> aMap){
    	StringBuffer qsb = null;

    	// only if map contains keys
    	if (aMap!=null && aMap.size()>0){
    		// iterate all entries
    		for (Iterator itr = aMap.entrySet().iterator(); itr.hasNext();) {
    			Map.Entry entry = (Map.Entry) itr.next();
    			try {
    				String tmp1 = URLEncoder.encode((String)entry.getKey(), HTTPConnectorUtil._UTF8_);
    				String tmp2 = URLEncoder.encode((String)entry.getValue(), HTTPConnectorUtil._UTF8_);
    				if (qsb==null){
    		    		qsb = new StringBuffer();
    				} else {
    					qsb.append("&");
    				}
    				qsb.append(tmp1).append("=").append(tmp2);
    			} catch (Exception e) { /* do nothing */}
    		}
    		return qsb.toString();
    	}
    	return null;
    }

    public static Map <String, String> manageHeadersFromNMR(Map <String, Object> nmrh, SpagicJettyHTTPExchange httpEx){
    	
    	// new map
    	Map <String, String> httpParameters = new HashMap <String, String> ();
    	
		for (Map.Entry<String, Object> e : nmrh.entrySet()) {
			// get key
			String aKey = e.getKey();
			// defensive test
			if (aKey!= null){
				// test if is a special param (begins with "HTTP-PRM-" or not)
				if (aKey.startsWith(prmPrefix)){
					// strip prefix
					String tmpPrmName = aKey.substring(prmPrefix.length());
					// put inside map
					httpParameters.put(tmpPrmName, (String)e.getValue());
				} else {
					// is a standard parameters
					httpEx.setRequestHeader(e.getKey(), (String)e.getValue());
				}
			}
		}
		return (httpParameters.size()>0)?httpParameters:null;
    }

    public static byte[] setParametersFromNMR (Map <String, String> prms, SpagicJettyHTTPExchange httpEx, String uri, 
    										   String prmStrategy, String multipartStrategy, String httpMethod,
    										   boolean isAttPresent) throws UnsupportedEncodingException{
    
		// append querystring if method GET + params via QueryString + params present
		// Strategy: 
		//   1) params==HTTPHeader   ---> set into httpExchange-RequestHeader
		//   2) params==QueryString && method=GET --> append to URL as querystring
		//   3) params==QueryString && method=PUT,DEL --> insert into body as querystring
		//   4) params==QueryString && method=POST && NO attachments --> insert into body as querystring
		//   5) params==QueryString && method=POST && attachments && multipart/form-data --> insert each parameter as mime part.
		//   6) params==QueryString && method=POST && attachments && multipart/related --> insert into bodypart as querystring.
	
		byte [] bb = null; 
	    	
		// case 1
		if (prmStrategy.equals(_HTTPHEADER)) {
			for (Map.Entry<String, String> e : prms.entrySet()) {
				// set param
				String aKey = e.getKey();
				httpEx.setRequestHeader(e.getKey(), e.getValue());
			}
		} 		
		// case 2
		if ( (prmStrategy.equals(_QUERYSTRING)) && (httpMethod.equals(HttpConstants.METHOD_GET)) ) {
			// get querystring from httpHeaders
			httpEx.setURL(uri + "?" + getEncodedQueryString(prms));
		} 	
		// case 3
		if ( (prmStrategy.equals(_QUERYSTRING)) && ( (httpMethod.equals(HttpConstants.METHOD_PUT)) || (httpMethod.equals(HttpConstants.METHOD_DELETE)) ) ){
			// get querystring from httpHeaders and prepare it as bytebuffer
			bb = getEncodedQueryString(prms).getBytes(HTTPConnectorUtil._UTF8_);
		} 		
		// case 4
		if ( (prmStrategy.equals(_QUERYSTRING)) && (httpMethod.equals(HttpConstants.METHOD_POST)) && (!isAttPresent) ){
			// get querystring from httpHeaders and prepare it as bytebuffer
			bb = getEncodedQueryString(prms).getBytes(HTTPConnectorUtil._UTF8_);
		} 		
		// case 5
			if ( (prmStrategy.equals(_QUERYSTRING)) && (httpMethod.equals(HttpConstants.METHOD_POST)) && 
					 (isAttPresent) && (multipartStrategy.equals(_MULTIPART_FORM)) ){
				// do nothing... all will be done later...        					
		} 		
		// case 6 
		if ( (prmStrategy.equals(_QUERYSTRING)) && (httpMethod.equals(HttpConstants.METHOD_POST)) && 
				 (isAttPresent) && (multipartStrategy.equals(_MULTIPART_RELATED)) ){
			// get querystring from httpHeaders and prepare it as bytebuffer
			bb = getEncodedQueryString(prms).getBytes(HTTPConnectorUtil._UTF8_);
		}
		
		return bb;
    }
    
    public static void prepareHttpJettyExchange (Map <String, String> prms, SpagicJettyHTTPExchange httpEx,  
			   String prmStrategy, String multipartStrategy, String httpMethod, String contentType, 
			   Message msg, byte[] buf) throws MessagingException, IOException{
    
    	boolean isAttPresent = false;
    	if ((msg.getAttachments()!= null && msg.getAttachments().size()>0)) 
    		isAttPresent = true;
    	
		// strategy: 
		//    1) attachments && POST && multipart/related
		//    2) (attachments and/or parameters) && POST && multipart/form-data
		//    3) rest of the world...
		
		// case 1:  attachments && POST && multipart/related
		if (isAttPresent && httpMethod.equals(HttpConstants.METHOD_POST) && multipartStrategy.equals(_MULTIPART_RELATED)){
			ByteArrayOutputStream baos = new ByteArrayOutputStream();
			try{
				writeMultipartAttachments(msg, baos, buf);
				httpEx.setRequestHeader(HttpHeaders.CONTENT_TYPE, (String)msg.getHeader(HTTPConnectorUtil.HTTP_CONTENT_TYPE));
				httpEx.setRequestContent(new ByteArrayBuffer(baos.toByteArray()));
			} finally {
				baos.close();
			}
		// case 2:  (attachments and/or parameters) && POST && multipart/form-data
		} else if ((isAttPresent || prms != null) &&	
				httpMethod.equals(HttpConstants.METHOD_POST) && multipartStrategy.equals(_MULTIPART_FORM)){
	
			ByteArrayOutputStream baos = new ByteArrayOutputStream();
			try{
				writeMultipartForm(msg, prms, baos, buf);
				httpEx.setRequestHeader(HttpHeaders.CONTENT_TYPE, (String)msg.getHeader(HTTPConnectorUtil.HTTP_CONTENT_TYPE));
				httpEx.setRequestContent(new ByteArrayBuffer(baos.toByteArray()));
			} finally {
				baos.close();
			}
	
		// case 3: otherwise
		} else {
			// set content type
			httpEx.setRequestHeader(HttpHeaders.CONTENT_TYPE, contentType);
			// set content
			if (buf!= null)
				// set body
				httpEx.setRequestContent(new ByteArrayBuffer(buf));
		}	
    }
    
    public static String readMultipartFormData(Message message, MimeMessage mime, Map<String,Object> paramToHeader) throws MessagingException, IOException {
    	  
    	  String bodyMessage = "";
    	  
    	  // Content must be ah InputStream
    	  Object content = mime.getInputStream(); // You could also
    	  if (content instanceof InputStream == false) {
    		  throw new UnsupportedOperationException("Expected a InputStream object but found a " + content.getClass());
    	  }
    	  
    	  // Convert the InputStream to MimeMultipart
    	  InputStream inputStream = (InputStream) content;
    	  ByteArrayDataSource inboundDataSource = new ByteArrayDataSource(inputStream, mime.getContentType());
    	  MimeMultipart multipart = new MimeMultipart(inboundDataSource); 
    	   
    	  // Analyze every single part of multipart
    	  for (int i = 0; i < multipart.getCount(); i++) {
    		   MimeBodyPart part = (MimeBodyPart) multipart.getBodyPart(i); 
    	 
	    	   Header tmp = null;
	    	   String name = "";
	    	   String value = "";
	    	   Enumeration<Header> enu1 = part.getAllHeaders();
	    	   String partName = "";
	    	   while (enu1.hasMoreElements()) {
		    	    tmp = (Header)enu1.nextElement();
		    	    name = tmp.getName();
		    	    value = tmp.getValue();
		    	    
		    	    // Get the name of parameter
		    	    if (name.toLowerCase().equals("content-disposition"))   {
		    	    	int idx1 = value.indexOf("name=\"");
		    	    	String subStr = value.substring(idx1+6);
		    	    	int idx2 = subStr.indexOf("\"");
		    	    	partName = subStr.substring(0, idx2);
		    	    }
	    	   }
		           
	    	   // if filename is not empty, this is a file
	    	   // You add it as Attachment
	    	   if (part.getFileName()!=null) {

	    		   message.addAttachment(partName, part.getDataHandler());
	
	    		// if filename is empty, this is a parameter or the body message
	    	   } else {
		    	    InputStream isPart = part.getInputStream();
		    	    String valuePart = HTTPConnectorUtil.convertStreamToString(isPart);
		    	    if (valuePart == null || valuePart.equals(""))
		    	    	isPart = new ByteArrayInputStream(valuePart.getBytes());
		    	    
		    	    // if part has no name, it is the body message
		    	    // otherwise it is a parameter
		    	    if (partName.equals("")) {
		    	    	
		    	    	// check if body is xml
		    	    	if(HTTPConnectorUtil.isXML(isPart)){
		    	    		// xml --> SET AS  Bodypart
		    	    		bodyMessage = valuePart;
		    	    	} else {
		    	    		// The part value is not a xml string
		    	    		bodyMessage = HTTPConnectorUtil.createXMLBody(valuePart);
		    	    	}
		    	    } else  {
		    	    	// Add the parameter to Header
		    	    	paramToHeader.put(partName, valuePart);
		    	    }
	    	   }
    	  }
    	  
    	  // if bodyMessage is empty
    	  // set a default xml
    	  if (bodyMessage.equals("")) bodyMessage = HTTPConnectorUtil.createXMLBody("");
    	  
    	  return bodyMessage;
    }    

    public static String readMultipartRelated(Message message, MimeMessage mime) throws MessagingException, IOException {
        
    	final Object content = mime.getContent();
    	String tmpXmlBody = null;
        
        if (content instanceof MimeMultipart == false) {
            throw new UnsupportedOperationException("Expected a javax.mail.internet.MimeMultipart object but found a " + content.getClass());
        }
        MimeMultipart multipart = (MimeMultipart) content;
        ContentType type = new ContentType(mime.getContentType());
        String contentId = type.getParameter("start");
        // Get request
        MimeBodyPart contentPart = null;
        if (contentId != null) {
            contentPart = (MimeBodyPart) multipart.getBodyPart(contentId);
        } else {
            for (int i = 0; i < multipart.getCount(); i++) {
              MimeBodyPart contentPart2 = (MimeBodyPart) multipart.getBodyPart(i);
              String contentType = contentPart2.getContentType();
              
              if (contentType.indexOf("xml") >= 0) {
                contentPart = contentPart2;
                break;
              }
            }
        }
        // Set new content
        tmpXmlBody = HTTPConnectorUtil.convertStreamToString(contentPart.getInputStream());
        
        // Get attachments
        for (int i = 0; i < multipart.getCount(); i++) {
            MimeBodyPart part = (MimeBodyPart) multipart.getBodyPart(i);
            if (part != contentPart) {
                String id = part.getContentID();
                if (id == null) {
                    id = "Part" + i;
                } else if (id.startsWith("<")) {
                    id = id.substring(1, id.length() - 1);
                }
                message.addAttachment(id, part.getDataHandler());
            }
        }
        
        return tmpXmlBody;
    }
    


}
