package org.spagic3.connectors.http.adapters;

import java.io.BufferedInputStream;
import java.io.InputStream;
import java.io.PrintWriter;
import java.io.StringWriter;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.dom4j.Document;
import org.dom4j.DocumentHelper;
import org.dom4j.Element;
import org.dom4j.io.SAXReader;
import org.dom4j.io.XMLWriter;
import org.spagic3.constants.SpagicConstants;
import org.spagic3.core.ExchangeUtils;
import org.spagic3.nmr.api.Exchange;
import org.spagic3.nmr.api.Message;
import org.spagic3.nmr.api.Pattern;
import org.spagic3.util.properties.PropertyConfigurator;

public class PlainHTTPInputProtocolAdapter implements IHTTPInputProtocolAdapter {

	
	public String getAdapterId() {
		return "PlainHTTP";
	}
	
	

	
	@Override
	public Exchange createExchange(HttpServletRequest request, String mep, String sender, String target,  PropertyConfigurator pc, Object binding) throws Exception  {
		Exchange exchange = null;
		
		if (mep.equalsIgnoreCase(SpagicConstants.IN_OUT_MEP)){
			exchange = ExchangeUtils.createExchange(sender,target, Pattern.InOut);
        }else if (mep.equalsIgnoreCase(SpagicConstants.IN_ONLY_MEP)){
        	exchange = ExchangeUtils.createExchange(sender,target, Pattern.InOnly);
        }
		
		Message in = exchange.getIn();
	    InputStream is = request.getInputStream();
	    is = new BufferedInputStream(is);
	    SAXReader reader = new SAXReader();
	    Document doc = reader.read(is);
	    in.setBody(doc.asXML());
	    exchange.setIn(in);
	    return exchange;
	}

	@Override
	public void sendAccepted(Exchange exchange, HttpServletRequest request,
			HttpServletResponse response, PropertyConfigurator pc, Object binding) {
		
		 response.setStatus(HttpServletResponse.SC_ACCEPTED);
		
	}

	@Override
	public void sendError(Exchange exchange, Exception error,
			HttpServletRequest request, HttpServletResponse response,PropertyConfigurator pc, Object binding) throws Exception {
		response.setStatus(HttpServletResponse.SC_INTERNAL_SERVER_ERROR);
        
        
		XMLWriter responseWriter = new XMLWriter(response.getWriter());
        Document doc = DocumentHelper.createDocument();
        
        Element root = doc.addElement("error");
        StringWriter sw = new StringWriter();
        PrintWriter pw = new PrintWriter(sw);
        error.printStackTrace(pw);
        pw.close();
        root.addText(sw.toString());
        responseWriter.write(doc);
        responseWriter.flush();
        responseWriter.close();
		
	}

	@Override
	public void sendFault(Exchange exchange, Message fault,
			HttpServletRequest request, HttpServletResponse response, PropertyConfigurator pc, Object binding) throws Exception {
		
		XMLWriter responseWriter = new XMLWriter(response.getWriter());
	   	 Document doc = DocumentHelper.parseText((String)fault.getBody());
	   	 responseWriter.write(doc);
	   	 responseWriter.flush();
	   	 responseWriter.close();
		
	}

	@Override
	public void sendOut(Exchange exchange, Message out,
			HttpServletRequest request, HttpServletResponse response, PropertyConfigurator pc, Object binding) throws Exception {
	
		XMLWriter responseWriter = new XMLWriter(response.getWriter());
   	 	Document doc = DocumentHelper.parseText((String)out.getBody());
   	 	responseWriter.write(doc);
   	 	responseWriter.flush();
   	 	responseWriter.close();
		
	}

	

}
