package org.spagic3.connectors.http.adapters;

import java.net.URI;

import org.mortbay.io.ByteArrayBuffer;
import org.mortbay.jetty.HttpHeaders;
import org.spagic3.nmr.api.Exchange;
import org.spagic3.nmr.api.Message;
import org.spagic3.nmr.api.Pattern;
import org.spagic3.nmr.api.Status;
import org.spagic3.util.properties.PropertyConfigurator;

/**
 * Default marshaler used for non-soap provider endpoints.
 *
 * @since 3.2
 */
public class PlainHTTPOutputProtocolAdapter implements IHTTPOutputProtocolAdapter {

	public String getAdapterId() {
		return "PlainHTTP";
	}
	
	private String DEFAULT_CONTENT_TYPE = "text/xml";
	

   
   

    protected String getLocationUri(Exchange exchange, Message inMsg, PropertyConfigurator pc)  {
       if (inMsg.getHeader("locationURI") != null){
    	   return (String)inMsg.getHeader("locationURI");
       }else{
    	   return pc.getString("locationURI");
       }
       
    }

    protected String getMethod(Exchange exchange, Message inMsg, PropertyConfigurator pc)  {
    	if (inMsg.getHeader("httpMethod") != null){
     	   return (String)inMsg.getHeader("httpMethod");
        }else{
        	 return pc.getString("httpMethod",null);
        }
    }

    protected String getContentType(Exchange exchange, Message inMsg, PropertyConfigurator pc)  {
    	if (inMsg.getHeader("content-type") != null){
      	   return (String)inMsg.getHeader("content-type");
         }else{
      	    return pc.getString("content-type", DEFAULT_CONTENT_TYPE);
         }
    }

    public void fillJettyExchange(Exchange exchange,
                              SpagicJettyHTTPExchange httpExchange, PropertyConfigurator pc, Object binding) {
        try{
        	Message inMsg = exchange.getIn(false);
        	httpExchange.setURL(getLocationUri(exchange, inMsg,pc));

        	// Temporary fix for bug in jetty-client 6.1.5
        	// http://fisheye.codehaus.org/browse/jetty-contrib/jetty/trunk/contrib/client/src/main/java/org/mortbay/jetty/client/HttpConnection.java?r1=374&r2=378
        	httpExchange.addRequestHeader(HttpHeaders.HOST_BUFFER, new ByteArrayBuffer(new URI(getLocationUri(exchange, inMsg,pc)).getHost()));

        	httpExchange.setMethod(getMethod(exchange, inMsg,pc));
        	httpExchange.setRequestHeader(HttpHeaders.CONTENT_TYPE, getContentType(exchange, inMsg,pc));
        	
        	/*
        	if (getHeaders() != null) {
        		for (Map.Entry<String, String> e : getHeaders().entrySet()) {
        			httpExchange.setRequestHeader(e.getKey(), e.getValue());
        		}
        	}
        	*/
        	
        	if (inMsg.getBody() != null) {
           
        		byte[] byteBuffer = ((String)inMsg.getBody()).getBytes();
        		httpExchange.setRequestContent(new ByteArrayBuffer(byteBuffer));
        	}
        }catch (Exception e) {
			throw new RuntimeException(e);
		}
    }

    public void handleResponse(Exchange exchange, SpagicJettyHTTPExchange httpExchange, PropertyConfigurator pc, Object binding) {
    	try{
    		int response = httpExchange.getResponseStatus();
    		if (response != org.mortbay.jetty.HttpStatus.ORDINAL_200_OK && response != org.mortbay.jetty.HttpStatus.ORDINAL_202_Accepted) {
            
        	if (exchange.getPattern() == Pattern.InOnly){
        		exchange.setError(new Exception("Invalid status response: " + response));
        	}
        	if (exchange.getPattern() == Pattern.InOut){
        		Message fault = exchange.getFault(true);
        		byte[] responze = httpExchange.getResponse();
        		fault.setBody((responze != null) ? new String(responze) : "");
        		exchange.setFault(fault);
        	}
        	
        } else  {
        	if (exchange.getPattern() == Pattern.InOut){
        		Message out = exchange.getOut();
        		out.setBody(new String(httpExchange.getResponse()));
        		exchange.setOut(out);
        	} else {
        		exchange.setStatus(Status.Done);
        	}
        }
    	}catch (Exception e) {
    		
			throw new RuntimeException(e);
		}
    }

    public void handleException(Exchange exchange, SpagicJettyHTTPExchange httpExchange, Throwable ex, PropertyConfigurator pc, Object binding) {
        exchange.setError((Exception)ex);
    }

}
