/*
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements.  See the NOTICE file distributed with
 * this work for additional information regarding copyright ownership.
 * The ASF licenses this file to You under the Apache License, Version 2.0
 * (the "License"); you may not use this file except in compliance with
 * the License.  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.spagic3.connectors.quartz.support;

import org.quartz.Job;
import org.quartz.JobExecutionContext;
import org.quartz.JobExecutionException;
import org.quartz.SchedulerException;
import org.spagic3.connectors.quartz.QuartzConnector;

/**
 * A Quartz Job which dispatches a message to a component.
 *
 * @version $Revision: 526407 $
 */
public class SpagicQuartzJob implements Job {
    
    public static final String COMPONENT_NAME = "org.spagic3.quartz.ComponentName";
    public static final String ENDPOINT_NAME = "org.spagic3.quartz.EndpointName";
    public static final String COMPONENT_NAME_PARAM_VALUE = "spagic3.component.quartz";
    
    public void execute(JobExecutionContext context) throws JobExecutionException {
        try {        	
        	
        	String connectorId = (String) context.getJobDetail().getJobDataMap().get(SpagicQuartzJob.ENDPOINT_NAME);
            if (connectorId == null) {
                throw new JobExecutionException("No property '" + SpagicQuartzJob.ENDPOINT_NAME + "' defined. Bad job data map");
            }
            QuartzConnector connector = (QuartzConnector) context.getScheduler().getContext().get(connectorId);
            if (connector == null) {
                throw new JobExecutionException("No quartz connector available for key: " + connectorId + "."
                        + " Bad job data map");
            }

            connector.onJobExecute(context);
        } catch (SchedulerException e) {
            throw new JobExecutionException(e);
        }
    }
    
}
