package org.spagic3.core;
import java.util.HashMap;
import java.util.Map;

import org.spagic3.constants.SpagicConstants;
import org.spagic3.nmr.api.Exchange;

/*
 * This object let you to set and get process variables from process instance bounded to the exchange
 * 
 */
public class ProcessContext {

	private Exchange exchange = null;


	public ProcessContext(Exchange exchange){
		this.exchange = exchange;
	}


	public void setVariables(Map<String, Object> variables){
		for ( String var : variables.keySet()){
			setVariable(var, variables.get(var));
		}
	}

	public Map<String, Object> getVariables(){
		Map<String, Object> resultingVariables  = new HashMap<String, Object>();
		Map<String, Object> exchangeProperties = exchange.getProperties();
		for ( String exchangePropertiesKey : exchangeProperties.keySet()){
			if (exchangePropertiesKey.startsWith(SpagicConstants.WF_VARIABLE_PREFIX)){
				// Saint - Wed Dec  2 10:15:12 2009
				//
				// we need to cut away just the prefix. We do no need
				// to seek where the part to remove is since we know
				// already that the string starts with
				// SpagicConstants.WF_VARIABLE_PREFIX (the test is
				// just over this comment, and we are in the code
				// executed when it's true) so we have to kill
				// SpagicConstants.WF_VARIABLE_PREFIX + 1 characters.
				String realWfvarName = exchangePropertiesKey.substring(SpagicConstants.WF_VARIABLE_PREFIX.length() + 1);
				//  </Saint>

				resultingVariables.put(realWfvarName, exchangeProperties.get(exchangePropertiesKey));
			}
		}
		return resultingVariables;
	}

	public Object getVariable(String variableName){

		String exchangePropForVariable = SpagicConstants.WF_VARIABLE_PREFIX + "." + variableName;
		return exchange.getProperty(exchangePropForVariable);

	}


	public  void setVariable(String variableName, Object variableValue){
		String exchangePropForVariable = SpagicConstants.WF_VARIABLE_PREFIX + "." + variableName;
		exchange.setProperty(exchangePropForVariable, variableValue);
	}
	
	public Long getProcessInstanceId() {
		Long processInstanceId = (Long) exchange.getProperty(SpagicConstants.WF_PROCESS_INSTANCE_ID_VARIABLE);
		return processInstanceId;
	}

}
