/**

    Copyright 2007, 2010 Engineering Ingegneria Informatica S.p.A.

    This file is part of Spagic.

    Spagic is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation; either version 3 of the License, or
    any later version.

    Spagic is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
    
**/
package org.spagic3.datasource;

import java.io.PrintWriter;
import java.sql.Connection;
import java.sql.SQLException;

import org.apache.commons.dbcp.BasicDataSource;
import org.apache.commons.dbcp.BasicDataSourceFactory;
import org.osgi.service.component.ComponentContext;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.spagic3.util.properties.PropertyConfigurator;



public class DataSource implements javax.sql.DataSource{
	
	private Logger logger = LoggerFactory.getLogger(DataSource.class);
	private javax.sql.DataSource internalDS = null;
	private BasicDataSourceFactory bdf = new BasicDataSourceFactory();
	
	protected void activate(ComponentContext componentContext){
		try{
			org.spagic3.util.properties.PropertyConfigurator propertyConfigurator = new PropertyConfigurator(componentContext.getProperties());	
			this.internalDS = bdf.createDataSource(propertyConfigurator.asProperties());
		}catch (Exception e) {
			throw new IllegalStateException("Cannot istantiate DataSource", e);
		}
	}
	
	
	
	@Override
	public Connection getConnection() throws SQLException {
		try{
			return internalDS.getConnection();
		}catch (Exception e) {
			logger.error(e.getMessage(), e);
			throw new SQLException(e);
		}
	}

	@Override
	public Connection getConnection(String username, String password)
			throws SQLException {
		
		return internalDS.getConnection(username, password);
	}

	@Override
	public PrintWriter getLogWriter() throws SQLException {
		
		return internalDS.getLogWriter();
	}

	@Override
	public int getLoginTimeout() throws SQLException {
		return internalDS.getLoginTimeout();
	}

	@Override
	public void setLogWriter(PrintWriter logWriter) throws SQLException {
		internalDS.setLogWriter(logWriter);
		
	}

	@Override
	public void setLoginTimeout(int loginTimeout) throws SQLException {
		internalDS.setLoginTimeout(loginTimeout);
		
	}

	@Override
	public boolean isWrapperFor(Class<?> arg0) throws SQLException {
		// TODO Auto-generated method stub
		return false;
	}

	@Override
	public <T> T unwrap(Class<T> arg0) throws SQLException {
		
		throw new SQLException("Not Implemented");
	}
	
	
	public String getDriverClassName(){
		return ((BasicDataSource)internalDS).getDriverClassName();
	}
	public String getUsername() {
		return ((BasicDataSource)internalDS).getUsername();
	}
	
	public String getUrl() {
		return ((BasicDataSource)internalDS).getUrl();
	}
	
	public String getPassword() {
		return ((BasicDataSource)internalDS).getPassword();
	}
	
	

	
	
	
}
