package org.spagic3.exchanges;

import java.util.concurrent.ConcurrentHashMap;

import org.osgi.framework.ServiceReference;
import org.osgi.service.component.ComponentContext;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.spagic3.constants.SpagicConstants;
import org.spagic3.nmr.api.Exchange;

public class ExchangeStoreManager implements IExchangeStoreManager {

	protected ConcurrentHashMap<String, ServiceReference> exchangeStore = new ConcurrentHashMap<String, ServiceReference>();
	private static Logger logger = LoggerFactory.getLogger(ExchangeStoreManager.class);
	private ComponentContext componentContext = null;
	
	protected void activate(ComponentContext componentContext) {
		logger.info(" ExchangeStoreManager - ACTIVATED");
		this.componentContext = componentContext;
	}

	protected void deactivate(ComponentContext componentContext) {
		logger.info(" ExchangeStoreManager - DEACTIVATED");
		
	}
	public Exchange deleteExchange(Long pid) {	
		return getExchangeStore().delete(pid);
	}

	
	public Exchange loadExchange(Long pid) {
		return getExchangeStore().load(pid);
	}

	
	public void storeExchange(Long pid, Exchange ex) {
		getExchangeStore().store(pid, ex);
	}

	protected IExchangeStore getExchangeStore(){
		String exchangeStoreSetted = System.getProperty(SpagicConstants.EXCHANGE_STORE);
		if (exchangeStoreSetted == null)
			exchangeStoreSetted = SpagicConstants.EXCHANGE_STORE_OSGI_PREF;
		
		ServiceReference sr = exchangeStore.get(exchangeStoreSetted);
		IExchangeStore selectedStore = (IExchangeStore)componentContext.locateService("exstores", sr);
		if (selectedStore == null)
			throw new RuntimeException(" Error in getting Exchange Store");
		
		return selectedStore;
	}
	
	
	public void addExStore(ServiceReference exstoreServiceReference) {
		
		String identifier = (String) exstoreServiceReference.getProperty("id");
		
		logger.info(" ExchangeStore ["+identifier+"] -- REGISTERED");
		
		
		exchangeStore.put(identifier, exstoreServiceReference);
		
	}

	public void removeDataSource(
			ServiceReference exstoreServiceReference) {

		String identifier = (String) exstoreServiceReference.getProperty("id");
		
		logger.info(" ExchangeStore ["+identifier+"] -- UNREGISTERED");
		exchangeStore.remove(identifier);
		

	}

	
}
