package org.spagic3.jmsconnectionfactory.jboss;

import java.util.Properties;

import javax.jms.JMSException;
import javax.naming.InitialContext;

import org.osgi.service.component.ComponentContext;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.spagic3.util.properties.PropertyConfigurator;

public class ConnectionFactory implements javax.jms.ConnectionFactory {
	
	private Logger logger = LoggerFactory.getLogger(ConnectionFactory.class);
	private javax.jms.ConnectionFactory cf = null;
	
	@Override
	public javax.jms.Connection createConnection() throws JMSException {
		logger.debug("Creating connection");
		return cf.createConnection();
	}

	@Override
	public javax.jms.Connection createConnection(String userName, String password)
			throws JMSException {
		logger.debug("Creating connection");
		return cf.createConnection(userName, password);
	}

	protected void activate(ComponentContext componentContext){
		try{
			PropertyConfigurator propertyConfigurator = new PropertyConfigurator(componentContext.getProperties());
			String providerUrl = propertyConfigurator.getString("java.naming.provider.url", "jnp://server:1099");
			String factoryUrlPkgs = propertyConfigurator.getString("java.naming.factory.url.pkgs", "org.jboss.naming");
			String factoryInitial = propertyConfigurator.getString("java.naming.factory.initial", "org.jnp.interfaces.NamingContextFactory");
			String jndiCFName = propertyConfigurator.getString("jndiConnectionFactoryName", "/ConnectionFactory");
						
			Properties props = new Properties();
			props.setProperty("java.naming.provider.url", providerUrl);
			props.setProperty("java.naming.factory.url.pkgs", factoryUrlPkgs);
			props.setProperty("java.naming.factory.initial", factoryInitial);

			InitialContext ctx = new InitialContext(props);
		    cf = (javax.jms.ConnectionFactory)ctx.lookup(jndiCFName);
		}catch (Exception e) {
			throw new IllegalStateException("Cannot istantiate ConnectionFactory", e);
		}
	}
		
}
