/**

    Copyright 2007, 2010 Engineering Ingegneria Informatica S.p.A.

    This file is part of Spagic.

    Spagic is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation; either version 3 of the License, or
    any later version.

    Spagic is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
    
**/
package org.spagic3.management.resources;

import java.io.File;
import java.io.FileWriter;
import java.io.PrintWriter;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.concurrent.atomic.AtomicReference;

import javax.sql.DataSource;

import org.dom4j.Document;
import org.dom4j.DocumentHelper;
import org.dom4j.Element;
import org.dom4j.Node;
import org.dom4j.XPath;
import org.dom4j.xpath.DefaultXPath;
import org.spagic3.constants.SpagicConstants;
import org.spagic3.core.BaseSpagicService;
import org.spagic3.datasource.IDataSourceManager;
import org.spagic3.nmr.api.Exchange;
import org.spagic3.nmr.api.Message;


public class DeployResourceService extends BaseSpagicService {
	

	protected final AtomicReference<IDataSourceManager> datasourceManager = new AtomicReference<IDataSourceManager>(null); 
	
	
	
	public IDataSourceManager getDataSourceManager() {
		return this.datasourceManager.get();
	}
	
	public void setDataSourceManager(IDataSourceManager dsm) {
		this.datasourceManager.set(dsm);
	}
	
	public void unsetDataSourceManager(IDataSourceManager dsManager) {
		this.datasourceManager.compareAndSet(dsManager, null);
	}
	@Override
	public boolean run(Exchange exchange, Message in, Message out)
			throws Exception {
		
	
		Document doc =  DocumentHelper.parseText((String)in.getBody());
		Document responseDoc = DocumentHelper.createDocument();
		Element rootResponse = responseDoc.addElement("DEPLOYMENT_RESULT");
		
		Map<String,String> namespaces = new HashMap<String, String>();
		namespaces.put("spagic","urn:eng:spagic"); 
		
		XPath xPathExpression = new DefaultXPath("spagic:payload/DEPLOY_DATASOURCES/DATASOURCE");
		xPathExpression.setNamespaceURIs(namespaces);

		List<Node> nodes = xPathExpression.selectNodes(doc);
		
		for (Node n : nodes){
			String id = n.valueOf("@name");
			Element rNode = rootResponse.addElement("DATASOURCE_DEPLOY_RESULT");
			rNode.addAttribute("name", id);
			
			DataSource ds = null;
			try{
				ds = datasourceManager.get().getDataSource(id);
				rNode.addAttribute("status", "already_deployed");
			}catch (Exception e) {
				boolean result = deployDSFile(id, getSpagicDSDescriptor(id, n));
				if (result)
					rNode.addAttribute("status", "successfully_deployed");
				else
					rNode.addAttribute("status", "not_deployed");
			}
			
		}
		out.setBody(responseDoc.asXML());
		return true;
	}
	public String getSpagicDSDescriptor(String id, Node n){
		StringBuffer sb = new StringBuffer();
		sb.append("<?xml version=\"1.0\" encoding=\"UTF-8\"?>"); 
		sb.append("<spagic:ds ");
		sb.append("    xmlns:spagic=\"urn:org:spagic3\"");
		sb.append("     xmlns=\"urn:org:spagic3\"");
		sb.append("    id=\""+id+"\">");
		     
		sb.append("<property name=\"driverClassName\" value=\""+n.valueOf("@driver")+"\"/>");
		sb.append("<property name=\"url\" value=\""+n.valueOf("@url")+"\"/>"); 
		sb.append("<property name=\"username\" value=\""+n.valueOf("@user")+"\"/>");
		sb.append("<property name=\"password\" value=\""+n.valueOf("@password")+"\"/>");
		
		sb.append("</spagic:ds>\n");
		
		return sb.toString();
	}
	public boolean  deployDSFile(String id, String xml) {
		try{
			String home = System.getProperty(SpagicConstants.SPAGIC_HOME_PROPERTY);
			String datasourcesFolder = home + File.separator + SpagicConstants.DATASOURCES_FOLDER;
		
			checkOrCreateFolder(datasourcesFolder);
		
			File f = new File (datasourcesFolder + File.separator + id + ".ds");
			PrintWriter pw = new PrintWriter(new FileWriter(f));
			pw.print(xml);
			pw.flush();
			pw.close();
			return true;
		}catch (Throwable e) {
			return false;
		}

		
	}
	
	private void checkOrCreateFolder(String fName) throws Exception {
		File f = new File(fName + File.separator);
		if (!f.exists())
			f.mkdirs();
		
	}
	


}
