/**

    Copyright 2007, 2010 Engineering Ingegneria Informatica S.p.A.

    This file is part of Spagic.

    Spagic is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation; either version 3 of the License, or
    any later version.

    Spagic is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
    
**/
package org.spagic3.management.resources;

import java.util.Set;
import java.util.concurrent.atomic.AtomicReference;

import javax.sql.DataSource;

import org.dom4j.Document;
import org.dom4j.DocumentHelper;
import org.dom4j.Element;
import org.spagic3.core.BaseSpagicService;
import org.spagic3.datasource.IDataSourceManager;
import org.spagic3.nmr.api.Exchange;
import org.spagic3.nmr.api.Message;


public class GetResourceService extends BaseSpagicService {
	
	
	protected final AtomicReference<IDataSourceManager> datasourceManager = new AtomicReference<IDataSourceManager>(null); 
	
	

	public IDataSourceManager getDataSourceManager() {
		return this.datasourceManager.get();
	}
	
	public void setDataSourceManager(IDataSourceManager dsm) {
		this.datasourceManager.set(dsm);
	}
	
	public void unsetDataSourceManager(IDataSourceManager dsManager) {
		this.datasourceManager.compareAndSet(dsManager, null);
	}
	@Override
	public boolean run(Exchange exchange, Message in, Message out)
			throws Exception {
		
		Document doc =  DocumentHelper.createDocument();
		Element rootElement = doc.addElement("DATASOURCES");
		Set<String> ids = getDataSourceManager().getRegisteredDatasourcesIDs();
		DataSource ds = null;
		for (String id : ids){
			ds = getDataSourceManager().getDataSource(id);
			Element dsElement = rootElement.addElement("DATASOURCE");
			dsElement.addAttribute("name", id);
			if (ds instanceof org.spagic3.datasource.DataSource){
				dsElement.addAttribute("driver",((org.spagic3.datasource.DataSource)ds).getDriverClassName());
				dsElement.addAttribute("user",((org.spagic3.datasource.DataSource)ds).getUsername());
				dsElement.addAttribute("url",((org.spagic3.datasource.DataSource)ds).getUrl());
				dsElement.addAttribute("password",((org.spagic3.datasource.DataSource)ds).getPassword());
			}
		}
		out.setBody(doc.asXML());
		return true;
	}

}
