/**

    Copyright 2007, 2010 Engineering Ingegneria Informatica S.p.A.

    This file is part of Spagic.

    Spagic is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation; either version 3 of the License, or
    any later version.

    Spagic is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
    
**/
package org.spagic3.monitoring;

import java.util.concurrent.BlockingQueue;
import java.util.concurrent.LinkedBlockingDeque;
import java.util.concurrent.atomic.AtomicReference;

import org.osgi.service.component.ComponentContext;
import org.osgi.service.event.Event;
import org.osgi.service.event.EventHandler;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.spagic3.databaseManager.IDatabaseManager;



public class MonitorService implements EventHandler {

	private static Logger logger = LoggerFactory.getLogger(MonitorService.class);
	@SuppressWarnings("unused")
	private ComponentContext componentContext = null;
	private final AtomicReference<IDatabaseManager> dbManager = new AtomicReference<IDatabaseManager>(null); 
	private BlockingQueue<Event> eventQueue = new LinkedBlockingDeque<Event>();
	private MonitorEventConsumer eventConsumer = null;

	protected void activate(ComponentContext componentContext) {
		logger.info(" Monitor Service - ACTIVATED");
		this.componentContext = componentContext;
		eventConsumer = new MonitorEventConsumer("MonitorQueueConsumer", eventQueue, getDatabaseManager());
		eventConsumer.start();
	}

	protected void deactivate(ComponentContext componentContext) {
		logger.info(" Monitor Service - DEACTIVATED");
	}

	@Override
	public void handleEvent(Event event) {
		try{
			eventQueue.put(event);
		}catch (InterruptedException e) {
			logger.error("Interrupted", e);
		}
	}
	
	
	public void setDatabaseManager(IDatabaseManager dbManager) {
		this.dbManager.set(dbManager);
	}

	public void unsetDatabaseManager(IDatabaseManager dbManager) {
		this.dbManager.compareAndSet(dbManager, null);		
	}
	
	public IDatabaseManager getDatabaseManager() {
		return this.dbManager.get();
	}
	

}
