package org.spagic3.profile.simple;


import java.io.File;
import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Timer;
import java.util.TimerTask;

import org.dom4j.Document;
import org.dom4j.Node;
import org.dom4j.io.SAXReader;
import org.osgi.service.component.ComponentContext;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.spagic.workflow.api.IProfileService;
import org.spagic3.startup.service.IStartedUpService;

public class DefaultProfileService implements IProfileService {

	private static Logger logger = LoggerFactory.getLogger(DefaultProfileService.class);
	protected List<String> dynamicRouting = new ArrayList<String>();
	protected String profilesFileName = "userprofile.xml";
	protected Timer timer = null;
	private Map<String, List<String>> userRolesMap = new HashMap<String, List<String>>();
	private File f = null;
	
	protected void loadProfiles(){
		try{
  			SAXReader reader = new SAXReader();
  			Document doc = reader.read(this.f);
  			
  			List<Node> users = doc.selectNodes("/AUTHORIZATIONS/ENTITIES/USERS/*");
  			List<Node> behaviours = doc.selectNodes("/AUTHORIZATIONS/RELATIONS/BEHAVIOURS/*");
  			synchronized (userRolesMap) {
  				userRolesMap = new HashMap<String, List<String>>();
	  			for (Node user: users) {
	  				String name = user.valueOf("@userID");
	  				List<String> roles = new ArrayList<String>();
	  				for (Node behaviour: behaviours) {
	  					String userid = behaviour.valueOf("@userID");
	  					String role = behaviour.valueOf("@roleName");
	  					if (userid.equals(name)) {
	  						roles.add(role);
	  					}
	  				}
	  				if (roles.size() > 0)
	  					userRolesMap.put(name, roles);
	  			}
  			}

    	}catch (Exception e) {
			e.printStackTrace();
		}
      }
	
	protected void activate(ComponentContext componentContext){
		String spagicHome = System.getProperty("spagic.home");
		
		String authFile = spagicHome + File.separator + profilesFileName;
		this.f = new File(authFile);
		loadProfiles();
		TimerTask task = new FileWatcher(  this.f ) {
		      protected void onChange( File file ) {
		    	  loadProfiles();
		      }
		};

		timer = new Timer();
		timer.schedule( task , new Date(), 5000 );
		
	}
	
	protected void deactivate(ComponentContext componentContext){
		timer.cancel();
	}
	
	
	
	
	public List<String> getUserRoles(String actorId) { 
		if (userRolesMap.containsKey(actorId)){
			return userRolesMap.get(actorId);
		}else{
			return new ArrayList<String>();
		}
		 
	}

	public List<String> getUsers() {
		List<String> users = new ArrayList<String>();
		users.addAll(userRolesMap.keySet());
		return users;
	}

	public void bindStartUpService(IStartedUpService startedUpService){
		logger.info("STARTUP SERVICE IS OK -> CAN START PROFILE SERVICE");	
	}
	
	public void unbindStartUpService(IStartedUpService startedUpService){
	}
	
}

