/*
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements.  See the NOTICE file distributed with
 * this work for additional information regarding copyright ownership.
 * The ASF licenses this file to You under the Apache License, Version 2.0
 * (the "License"); you may not use this file except in compliance with
 * the License.  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
/* This file contains code modified from Apache code */
package org.spagic3.soap.ws.addressing;

import javax.xml.namespace.QName;

import org.spagic3.common.util.DOMUtil;
import org.spagic3.common.util.WSAddressingConstants;
import org.spagic3.soap.bindings.soap.SoapInterceptor;
import org.spagic3.soap.core.AbstractInterceptor;
import org.spagic3.soap.util.DomUtil;
import org.w3c.dom.Element;
import org.w3c.dom.DocumentFragment;
import org.w3c.dom.Document;


public abstract class AbstractWsAddressingInterceptor extends AbstractInterceptor implements SoapInterceptor {

    private final WsAddressingPolicy policy;
    private final boolean server;

    protected AbstractWsAddressingInterceptor(WsAddressingPolicy policy, boolean server) {
        this.policy = policy;
        this.server = server;
    }

    public WsAddressingPolicy getPolicy() {
        return policy;
    }

    public boolean isServer() {
        return server;
    }

    protected boolean isWSANamespace(String ns) {
        return WSAddressingConstants.WSA_NAMESPACE_200303.equals(ns) ||
               WSAddressingConstants.WSA_NAMESPACE_200403.equals(ns) ||
               WSAddressingConstants.WSA_NAMESPACE_200408.equals(ns) ||
               WSAddressingConstants.WSA_NAMESPACE_200508.equals(ns);
    }

    protected String getHeaderText(Object header) {
        Element el = (Element) ((DocumentFragment) header).getFirstChild();
        return DOMUtil.getElementText(el);
    }

    protected DocumentFragment createHeader(QName name, String value) throws Exception {
        Document doc = DomUtil.createDocument();
        DocumentFragment df = doc.createDocumentFragment();
        Element el = doc.createElementNS(name.getNamespaceURI(), getQualifiedName(name));
        el.appendChild(doc.createTextNode(value));
        df.appendChild(el);
        return df;
    }

    /**
     * Gets the QName prefix.  If the QName has no set prefix, the specified default prefix will be used.
     */
    protected String getPrefix(QName qname, String defaultPrefix) {
    	String prefix = qname.getPrefix();
    	if(null == prefix || "".equals(prefix)) {
    		prefix = defaultPrefix;
    	}

    	return prefix;
    }

    protected String getQualifiedName(QName qname) {
    	String name = qname.getLocalPart();

    	String prefix = qname.getPrefix();
    	if(null != prefix && (!"".equals(prefix))) {
    		name = prefix + ":" + name;
    	}

    	return name;
    }

}
